#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config, get_data_from_script_output
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from time import sleep

def cleanup_storage():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    ag_name_2 = read_var("LSM_AG_NAME_2")
    # fs_name = read_var("LSM_FS_NAME")
    # fs_cloned_name = read_var("LSM_FS_CLONED_NAME")
    # fs_snap_name = read_var("LSM_FS_SNAP_NAME")
    # vol_name = read_var("LSM_VOL_NAME")
    # vol_name_2 = read_var("LSM_VOL_NAME_2")
    # vol_rep_name = read_var("LSM_VOL_REP_NAME")

    config = list(yield_lsm_config())[0]
    if not config['protocol']:
        print("This is just local disk test, no need to clean up storage.")
        return errors

    lsm = LibStorageMgmt(disable_check=True, **config)

    ret, data = atomic_run("Listing access groups",
                           command=lsm.list,
                           lsm_type="ACCESS_GROUPS",
                           return_output=True,
                           script=True,
                           errors=errors)
    ags = get_data_from_script_output(data)
    for ag in [ag_name_2, ag_name]:
        ag_id = None
        if ags is None:
            break
        for ags_id in ags:
            if ags[ags_id]['Name'] == ag:
                ag_id = ags_id
                break
        if ag_id is None:
            print("INFO: Could not find ag %s, continuing." % ag)
            continue
        atomic_run("Removing AG '%s'." % ag,
                   command=lsm.access_group_delete,
                   ag=ag_id,
                   force=True,
                   errors=errors)
    # give some time to print all data
    sleep(1)

    return errors


if __name__ == "__main__":
    errs = cleanup_storage()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI
from libsan.host.linux import is_module_loaded


def test_python_kmod():
    errors = []
    targetcli = TargetCLI()

    atomic_run("INFO: Running targetcli get command.",
               group="global",
               command=targetcli.get,
               errors=errors
               )

    atomic_run("INFO: Checking if module is loaded",
               module_name="target_core_mod",
               command=is_module_loaded,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = test_python_kmod()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""mfs_lock.py: Module to handle storage reservation using NFS."""

import re  # regex
import os
import libsan.host.linux
import libsan.sanmgmt
import libsan.host.iscsi as iscsi
import stqe.host.beaker as beaker
from libsan.host.cmdline import run

"""
The idea is kind complex.
1. Data structures
1.1 Queue Data:
All servers that want use a storage device need to request it.
The request is added to the queue data.
During the request it is informed if host wants to use the storage device exclusively or shared
1.2 Lock Data:
1.2.1 Lock data contains all hosts that are locked the storage_device.
1.2.2 Lock data can have more than 1 entry there if they are all shared
1.2.3 Exclusive lock can only be added when lock data is empty, only 1 entry per time

2. Moving data from Queue to Lock data
2.1 If there is exclusive lock on queue, try to add it to lock data
2.1.1 If lock data is empty, lock is added
2.1.2 If lock data is not empty the exclusive request remains on queue until Lock data is free
2.2 If there is shared request and no exclusive request
2.2.1 There is no exclusive lock on Lock data, shared lock is added to Lock data
2.2.2 If Lock data has exclusive lock, need to wait lock data to be empty

3. Need to guaranty exclusive access to NFS files
3.1 To do that there is an update_token. The host that gets it can change lock data and queue data
3.2 Prior any change on Data structures we need to make sure we got the update_token

4. Usage:
4.1 nfs_lock = libsan.host.nfs_lock.nfs_lock("server","share","mount_point","storage_name")
4.2 nfs_lock.request_lock("shared")
4.3 nfs_lock.get_lock() #It blocks until get the lock
4.4 nfs_lock.release_lock("shared")
"""

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _read_file(file_path):
    """
    """
    while True:
        try:
            if not os.path.isfile(file_path):
                return None
            f = open(file_path)
            break
        except Exception as e:
            print(e)
            _print("FAIL: _read_file() - Could not open %s" % file_path)
            libsan.host.linux.sleep(5)
    if not f:
        _print("FAIL: _read_file() - Could not open %s" % file_path)
        return None

    try:
        file_data = f.read()
    except Exception as e:
        print(e)
        # Maybe due Stale IO, like the path has been deleted...
        _print("FAIL: _read_file() - Could not read %s" % file_path)
        f.close()
        return None
    f.close()
    if not file_data:
        return None
    file_lines = file_data.split("\n")
    if not file_lines:
        return None

    return_lines = []
    # Remove empty lines
    for line in file_lines:
        if line != "":
            return_lines.append(line)

    return return_lines


def _delete_item_queue_file(file_path, item_str):
    """
    """
    _print("DEBUG: Deleting %s from %s" % (item_str, file_path))

    queue_info = _read_file(file_path)
    if not queue_info:
        _print("FAIL: Could not read %s" % file_path)
        return False

    try:
        f = open(file_path, "w")
    except Exception as e:
        print(e)
        _print("FAIL: _delete_item_queue_file() - Could not open %s" % file_path)
        return False
    for line in queue_info:
        if line != item_str:
            f.write("%s\n" % line)
    f.close()
    return True


def setup_nfs_lock_for_mpath(mpath_name):
    """
    To avoid race condition when performing disruptive action on Storage array
    We use NFS as lock mechanism
    Return:
    \tNFS lock object
    """
    if not mpath_name:
        _print("FAIL: setup_nfs_lock_for_mpath() - requires mpath_name as parameter")
        return None

    obj_libsan = libsan.sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_libsan:
        _print("FAIL: setup_nfs_lock_for_mpath() - Could not create libsan for %s" % mpath_name)
        return None

    general_conf_dict = None
    if "general" in obj_libsan.san_conf_dict:
        general_conf_dict = obj_libsan.san_conf_dict["general"]

    if not general_conf_dict:
        _print("FAIL: Could not set NFS lock. It is not configured on san config file")
        print(obj_libsan.san_conf_dict)
        return None

    if "nfs_server" not in general_conf_dict:
        _print("WARN: setup_nfs_lock_for_mpath() - nfs_server is not "
               "configured on SAN config file, under general section")
        print(general_conf_dict)
        return None

    if "nfs_share" not in general_conf_dict:
        _print("WARN: setup_nfs_lock_for_mpath() - nfs_share is not configured "
               "on SAN config file, under general section")
        print(general_conf_dict)
        return None

    if "nfs_mount_point" not in general_conf_dict:
        _print("WARN: setup_nfs_lock_for_mpath() - nfs_mount_point is not "
               "configured on SAN config file, under general section")
        print(general_conf_dict)
        return None

    sa = obj_libsan.get_sa_self()
    if not sa:
        _print("FAIL: setup_nfs_lock_for_mpath() - Could not find Storage "
               "Array managed by libsan")
        return None

    if "lock_array_name" not in sa:
        _print("FAIL: setup_nfs_lock_for_mpath() - lock_array_name is not defined "
               "under array section on SAN config file")
        print(sa)
        return None
    lock_obj = NfsLock(general_conf_dict["nfs_server"], general_conf_dict["nfs_share"],
                       general_conf_dict["nfs_mount_point"], "%s" % sa["lock_array_name"])
    mount_point = general_conf_dict["nfs_mount_point"]
    if not os.path.isdir(mount_point):
        if not libsan.host.linux.mkdir(mount_point):
            _print("FAIL: Could not create %s" % mount_point)
            return None

    libsan.host.linux.umount(general_conf_dict["nfs_mount_point"])
    if not lock_obj.mount_lock():
        return None
    return lock_obj


def setup_nfs_lock_lun(obj_sangmtm, wwid, nfs_lock_type):
    """
    Lock whole storage array with 'shared' lock and lock LUN on the array with nfs_lock_type lock
    :param obj_sangmtm: libsan.sangmtm.SanMgmt dict with information about the storage array
    :param wwid: wwid of LUN to lock
    :param nfs_lock_type: lock type ['shared', 'exclusive'] of the LUN
    :return: True | False
    """
    general_conf_dict = obj_sangmtm.san_conf_dict["general"]
    sa = obj_sangmtm.get_sa_self()
    if not sa:
        _print("FAIL: setup_nfs_lock_lun() - there is no array configured for given sanmgmt config.")
        return False

    # create shared lock on storage array
    _print("INFO: setup_nfs_lock_lun() - creating 'shared' lock on storage array")
    lock_obj_sa = NfsLock(general_conf_dict["nfs_server"], general_conf_dict["nfs_share"],
                          general_conf_dict["nfs_mount_point"], "%s" % sa["lock_array_name"])

    mount_point = general_conf_dict["nfs_mount_point"]
    if not os.path.isdir(mount_point):
        if not libsan.host.linux.mkdir(mount_point):
            _print("FAIL: Could not create %s" % mount_point)
            return None
    libsan.host.linux.umount(general_conf_dict["nfs_mount_point"])
    if not lock_obj_sa.mount_lock():
        return None

    # add shared lock to the storage array as we are locking just one LUN
    if not lock_obj_sa.request_lock("shared"):
        _print("FAIL: setup_nfs_lock_lun() - could not request lock on storage array.")
        return False
    # need to wait on lock before we can proceed
    _print("INFO: Storage array lock request added to queue, starting to wait on lock.")
    if not lock_obj_sa.get_lock():
        _print("FAIL: setup_nfs_lock_lun() - could not get lock on storage array.")
        return False
    _print("INFO: setup_nfs_lock_lun() - storage array locked - 'shared'")

    # add lock to the LUN
    _print("INFO: setup_nfs_lock_lun() - creating '%s' lock for LUN '%s'" % (nfs_lock_type, wwid))
    lock_obj_lun = NfsLock(general_conf_dict["nfs_server"], general_conf_dict["nfs_share"],
                           general_conf_dict["nfs_mount_point"], "%s" % sa["lock_array_name"] + "/" + wwid)
    if not lock_obj_lun.request_lock(nfs_lock_type):
        _print("FAIL: setup_nfs_lock_lun() - could not request lock on LUN.")
        return False

    _print("INFO: NFS lock request added to queue, starting to wait on lock.")
    if not lock_obj_lun.get_lock():
        _print("FAIL: setup_nfs_lock_lun() - could not get lock on LUN.")
        return False
    _print("INFO: setup_nfs_lock_lun() - LUN locked - '%s'" % nfs_lock_type)
    return True


def release_nfs_lock_lun(obj_sanmgmt, wwid, nfs_lock_type):
    """
    Relase lock from whole storage array and LUN on the array with nfs_lock_type lock
    :param obj_sangmtm: libsan.sangmtm.SanMgmt dict with information about the storage array
    :param wwid: wwid of LUN to unlock
    :param nfs_lock_type: lock type ['shared', 'exclusive'] of the LUN
    :return: True | False
    """
    general_conf_dict = obj_sanmgmt.san_conf_dict["general"]
    sa = obj_sanmgmt.get_sa_self()
    if not sa:
        _print("FAIL: setup_nfs_lock_lun() - there is no array configured for given sanmgmt config.")
        return False

    # release lock from the LUN
    _print("INFO: release_nfs_lock_lun - releasing LUN '%s' lock" % nfs_lock_type)
    lock_obj_lun = NfsLock(general_conf_dict["nfs_server"], general_conf_dict["nfs_share"],
                           general_conf_dict["nfs_mount_point"], "%s" % sa["lock_array_name"] + "/" + wwid)
    if not lock_obj_lun.release_lock(nfs_lock_type):
        _print("FAIL: release_nfs_lock_lun() - could not release lock on LUN '%s', trying again." % wwid)
        lock_obj_lun.release_lock(nfs_lock_type, 10)
        # do not return here, we still need to clear lock on storage array
    _print("INFO: release_nfs_lock_lun() - LUN lock '%s' released" % nfs_lock_type)

    # release shared lock on storage array
    _print("INFO: release_nfs_lock_lun() - removing 'shared' lock on storage array")
    lock_obj_sa = NfsLock(general_conf_dict["nfs_server"], general_conf_dict["nfs_share"],
                          general_conf_dict["nfs_mount_point"], "%s" % sa["lock_array_name"])
    if not lock_obj_sa.release_lock("shared"):
        _print("FAIL: release_nfs_lock_lun() - could not release lock on storage array, trying again.")
        if not lock_obj_sa.release_lock("shared", 10):
            return False
    _print("INFO: release_nfs_lock_lun() - storage array lock 'shared' released")
    return True


def _get_iscsi_params(sid):
    """
    Loads params from iscsi to sanmgmt dict and returns it.
    :param iqn: iscsi iqn to get params about
    :return: dict loaded sanmgmgt | None
    """
    iscsi_session = iscsi.query_iscsi_session(sid)
    if not iscsi_session:
        _print("FAIL: setup_nfs_lock_for_iscsi() - could not fid iscsi session sid '%s'" % sid)
        return None

    h_iqn = iscsi_session["h_iqn"]
    t_iqn = iscsi_session["t_iqn"]
    iface_mac = iscsi_session["mac"]
    map_info = {"t_iqn": t_iqn, "h_iqn": h_iqn}

    obj_libsan = libsan.sanmgmt.SanMgmt()
    obj_libsan.map_info(map_info)
    obj_libsan.h_iqns([h_iqn])
    obj_libsan.t_iqns([t_iqn])
    obj_libsan.macs([iface_mac])
    obj_libsan.load_conf()
    return obj_libsan


def setup_nfs_lock_iscsi(disk_name, nfs_lock_type="exclusive"):
    """
    Create lock on iscsi LUN and 'shared' lock on its storage array.
    :param disk_name: disk to lock (for example 'sdd')
    :param nfs_lock_type: lock type ['shared', 'exclusive'] of the LUN
    :return: True | False
    """

    if not disk_name:
        _print("FAIL: setup_nfs_lock_iscsi() - requires disk_name as parameter")
        return False

    sid = libsan.host.iscsi.get_session_id_from_disk(disk_name)
    wwid = libsan.host.scsi.wwid_of_disk(disk_name)
    _print("INFO: setup_nfs_lock_iscsi() - locking wwid '%s' on sid '%s'" % (wwid, sid))

    obj_libsan = _get_iscsi_params(sid)
    if not obj_libsan:
        _print("FAIL: setup_nfs_lock_iscsi() - could not find sid %s in iscsi sessions" % sid)
        return False

    if not setup_nfs_lock_lun(obj_libsan, wwid, nfs_lock_type):
        _print("FAIL: setup_nfs_lock_iscsi() - could not setup lock for iscsi LUN.")
        return False
    return True


def release_nfs_lock_iscsi(disk_name, nfs_lock_type="exclusive"):
    """
    Removes nfs_lock_type lock on iscsi LUN and 'shared' lock on storage array.
    :param iqn: IQN of LUN to lock
    :param nfs_lock_type: lock type ['shared', 'exclusive']
    :return: True | False
    """
    if not disk_name:
        _print("FAIL: release_nfs_lock_iscsi() - requires iqn as parameter")
        return None

    sid = libsan.host.iscsi.get_session_id_from_disk(disk_name)
    wwid = libsan.host.scsi.wwid_of_disk(disk_name)

    obj_libsan = _get_iscsi_params(sid)
    if not obj_libsan:
        _print("FAIL: release_nfs_lock_iscsi() - could not find sid '%s' in iscsi sessions" % sid)
        return False

    if not release_nfs_lock_lun(obj_libsan, wwid, nfs_lock_type):
        _print("FAIL: release_nfs_lock_iscsi() - could not release lock for iscsi LUN.")
        return False
    return True


class NfsLock(object):
    """
    Class to manage NFS Lock
    """
    _nfs_server = None
    _nfs_export = None
    _mount_point = None
    _device_name = None
    _directory_path = None

    _monitor_pid = None

    _queue_file = "queue_file"
    _queue_file_path = None

    _lock_file = "lock_file"
    _lock_file_path = None

    # As NFS does not handle well multiple access to a file
    # We will try to make sure only 1 server has access to the lock info at same time
    _update_token_file = "update_token"
    _update_token_file_path = None

    # Queue has hostname and lock_type info
    _queue_format_regex = re.compile(r"^(\S+) lock_type: (\S+) time: (\S+) task_id: (\S+)")
    _lock_format_regex = re.compile(r"^(\S+)\s(\S+)\s(\S+)\s(\S+)")

    supported_lock_types = ["shared", "exclusive"]

    def __init__(self, nfs_server, nfs_export, mount_point, storage_name):
        """
        """
        if not nfs_server:
            raise ValueError("FAIL: __init__() - requires nfs_server parameterr")

        if not nfs_export:
            raise ValueError("FAIL: __init__() - requires nfs_export parameter")

        if not mount_point:
            raise ValueError("FAIL: __init__() - requires mount_point parameter")

        if not storage_name:
            raise ValueError("FAIL: __init__() - requires storage_name parameter")

        self._nfs_server = nfs_server
        self._nfs_export = nfs_export
        self._mount_point = mount_point
        self._device_name = storage_name

        self._directory_path = "%s/%s" % (mount_point, storage_name)

        self._queue_file_path = "%s/%s" % (self._directory_path, self._queue_file)
        self._lock_file_path = "%s/%s" % (self._directory_path, self._lock_file)
        self._update_token_file_path = "%s/%s" % (self._directory_path, self._update_token_file)

    def mount_lock(self):
        """
        Mount NFS lock export
        """
        src = "%s:%s" % (self._nfs_server, self._nfs_export)
        if not libsan.host.linux.mount(src, self._mount_point, fs="nfs"):
            _print("FAIL: Could not mount %s:%s" % (self._nfs_server, self._nfs_export))
            return False
        return True

    def umount_lock(self):
        """
        umount NFS lock export
        """
        if not libsan.host.linux.umount(self._mount_point):
            _print("FAIL: Could not umount %s" % self._mount_point)
            return False
        return True

    #   ########## UPDATE TOKEN ############
    def get_update_token(self):
        """
        Waits until we get the right to update the NFS folder
        """
        _print("INFO: Waiting to get update token")
        if not os.path.isdir(self._directory_path):
            if not libsan.host.linux.mkdir(self._directory_path):
                _print("FATAL: Could not create directory: %s" % self._directory_path)
                return False
            # Sync after creating the directory
            libsan.host.linux.sync(self._directory_path)

        got_token = False
        update_token_regex = re.compile(r"^(\S+) (\S+)")
        count = 0
        previous_token = None
        elapsed_time = 0
        while not got_token:
            if not os.path.isfile(self._update_token_file_path):
                # It seems we can get access to update NFS directory
                token_info = "%s %s" % (libsan.host.linux.hostname(),
                                        libsan.host.linux.time_stamp(in_seconds=True))
                run("echo %s >> %s" % (token_info, self._update_token_file_path))
                libsan.host.linux.sync(self._directory_path)
                libsan.host.linux.sleep(10)
                libsan.host.linux.sync(self._directory_path)
                token = _read_file(self._update_token_file_path)
                if not token or (len(token) > 1):
                    _print("FAIL: It seems we got concurrent access to update token. "
                           "Deleting it...")
                    run("rm -f %s" % self._update_token_file_path)
                    libsan.host.linux.sync(self._directory_path)
                    continue
                m = update_token_regex.match(token[0])
                if not m:
                    _print("FAIL: %s is not valid token" % token[0])
                    run("rm -f %s" % self._update_token_file_path)
                    libsan.host.linux.sync(self._directory_path)
                _print("INFO: We got the update token!")
                return token
            else:
                token = _read_file(self._update_token_file_path)
                if not token or (previous_token != token):
                    # token changed, we should refresh counter
                    elapsed_time = 0
                    previous_token = token

                if elapsed_time > 60:
                    _print("FAIL: timeout expired for update token. Deleting it...")
                    run("rm -f %s" % self._update_token_file_path)
                    libsan.host.linux.sync(self._directory_path)

                if count >= 5:
                    _print("INFO: Still waiting for update token...")
                    count = 0
                # Sleep a bit before checking for the token again
                libsan.host.linux.sleep(1)
                count += 1
                elapsed_time += 1

    def release_update_token(self):
        """
        Release the update token
        """
        libsan.host.linux.sync(self._directory_path)
        if os.path.isdir(self._directory_path):
            if os.path.isfile(self._update_token_file_path):
                run("rm -f %s" % self._update_token_file_path)
                libsan.host.linux.sync(self._directory_path)
                return True
        return False

    #       ############# QUEUE ########################

    def _add_queue(self, host, lock_type):
        """
        Add lock request to the queue
        """

        if not os.path.isdir(self._directory_path):
            if not libsan.host.linux.mkdir(self._directory_path):
                _print("FAIL: _add_queue() - Could not create %s" % self._directory_path)
                return False

        if not host or not lock_type:
            _print("FAIL: _add_queue() - requires host and lock_type parameters")
            return False

        if not libsan.host.linux.sync(self._directory_path):
            _print("FAIL: Could not sync %s" % self._directory_path)
            return None

        queue_data = _read_file(self._queue_file_path)
        beaker_task_id = None
        if "TASKID" in os.environ:
            beaker_task_id = os.environ['TASKID']
        if not beaker_task_id:
            # TODO
            beaker_task_id = "manual"
        device_info = "%s lock_type: %s time: %s task_id: %s" % (host, lock_type,
                                                                 libsan.host.linux.time_stamp(in_seconds=True),
                                                                 beaker_task_id)
        if queue_data:
            for queue_device in queue_data:
                m = self._queue_format_regex.match(queue_device)
                if not m:
                    _print("FAIL: '%s' does not match a queue entry format" % queue_device)
                    _delete_item_queue_file(self._queue_file_path, queue_device)
                    continue
                if m.group(1) == host:
                    _print("INFO: Deleting previous entry for host %s (%s)" % (host, queue_device))
                    _delete_item_queue_file(self._queue_file_path, queue_device)

        # Open file for write with append
        f = open(self._queue_file_path, "a")
        if not f:
            _print("FAIL: _add_queue() - Could not open %s" % self._queue_file_path)
            return False

        try:
            f.write("%s\n" % device_info)
            f.close()
        except Exception as e:
            print(e)
            _print("FAIL: _add_queue() - Could not add %s to queue" % host)
            f.close()
            return False

        if not libsan.host.linux.sync(self._directory_path):
            _print("FAIL: Could not sync %s" % self._directory_path)
            return None
        return True

    def _dequeue(self):
        """
        Remove a lock request from queue
        """
        _print("INFO: Updating queue and lock lists")
        if not os.path.isdir(self._directory_path):
            # It seems no lock has ever been queued to this device
            _print("INFO: There is not directory %s" % self._directory_path)
            return None

        if not os.path.isfile(self._queue_file_path):
            _print("DEBUG: _dequeue() -  %s does not exist" % self._queue_file_path)
            return None

        if not libsan.host.linux.sync(self._directory_path):
            _print("FAIL: Could not sync %s" % self._directory_path)
            return None

        queue_devices = _read_file(self._queue_file_path)
        if not queue_devices:
            _print("DEBUG: _dequeue() - there is no host queued for %s" % self._device_name)
            return None

        # list of devies that should be add to lock file and removed from queue
        devices_2_lock = []
        # If first entry is shared we can lock more devices that also
        # request for shared access
        # if first entry is exclusive we just add it to lock
        _print("DEBUG: _dequeue() - queue devices:")
        print(queue_devices)

        # Check if there is requests queued for long time
        # If there is the queue will contain only these requests
        wait_long_devs = []
        for queue_device in queue_devices:
            _print("DEBUG: _dequeue() - checking if request is waiting for long time in queue: '%s'" % queue_device)
            m = self._queue_format_regex.match(queue_device)
            if not m:
                _print("FAIL: '%s' does not match a queue entry format" % queue_device)
                _delete_item_queue_file(self._queue_file_path, queue_device)
                queue_devices.remove(queue_device)
                continue
            current_timestamp = int(libsan.host.linux.time_stamp(in_seconds=True))
            timeout = int(m.group(3)) + (60 * 60)
            # If a request has not been queued for more than 1h it is not queued long enough
            # to get higher priority
            if current_timestamp < timeout:
                continue
            processing_lock_type = m.group(2)
            if processing_lock_type == "exclusive":
                # Just add the exclusive device
                wait_long_devs = [queue_device]
                break
            wait_long_devs.append(queue_device)
        if wait_long_devs:
            queue_devices = wait_long_devs
        # Check if there is any server waiting for exclusive access
        # They have priority
        for queue_device in queue_devices:
            _print("DEBUG: _dequeue() - checking for exclusive lock in queue: '%s'" % queue_device)
            m = self._queue_format_regex.match(queue_device)
            if not m:
                _print("FAIL: '%s' does not match a lock entry format" % queue_device)
                _delete_item_queue_file(self._queue_file_path, queue_device)
                queue_devices.remove(queue_device)
                continue
            processing_lock_type = m.group(2)
            if processing_lock_type == "exclusive":
                devices_2_lock.append(queue_device)
                break
        # If there is no exclusive request
        if not devices_2_lock:
            for queue_device in queue_devices:
                _print("DEBUG: _dequeue() - checking for shared lock in queue: '%s'" % queue_device)
                m = self._queue_format_regex.match(queue_device)
                if not m:
                    _print("FAIL: '%s' does not match a lock entry format" % queue_device)
                    _delete_item_queue_file(self._queue_file_path, queue_device)
                    queue_devices.remove(queue_device)
                    continue
                processing_lock_type = m.group(2)
                if processing_lock_type == "shared":
                    devices_2_lock.append(queue_device)
                    continue

        if not devices_2_lock:
            _print("DEBUG: _dequeue() - there is no lock device pending for %s" % self._device_name)
            return None

        for device in devices_2_lock:
            m = self._queue_format_regex.match(device)
            if not m:
                _print("FAIL: _dequeue() - device %s is not on proper format" % device)
                self.show_queue()
                return None
            if not self._add_lock(m.group(1), m.group(2), m.group(4)):
                _print("FAIL: _dequeue() - Could not add %s to lock file" % device)
                return None
            # device success fully added to lock, remove it from queue
            _delete_item_queue_file(self._queue_file_path, device)

        if not libsan.host.linux.sync(self._directory_path):
            _print("FAIL: Could not sync %s" % self._directory_path)
            return None
        _print("DEBUG: _dequeue() - Servers locked using %s are:" % self._device_name)
        self.show_lock()
        return devices_2_lock

    #   ########## LOCK ###################
    def show_queue(self):
        """
        Show lock queue
        """
        _print("INFO: ### Showing NFS queue data #########")
        cmd = "cat %s " % self._queue_file_path
        ret, output = run(cmd, return_output=True, verbose=False)
        print(output)
        _print("INFO: ### End NFS queue data #########")
        return True

    def request_lock(self, lock_type):
        """
        Add this lock request to lock queue
        """
        host = libsan.host.linux.hostname()
        _print("INFO: Requesting %s lock for %s" % (lock_type, host))

        if not lock_type:
            _print("FAIL: request_lock() - requires lock_type parameters")
            return False

        if lock_type not in self.supported_lock_types:
            _print("FAIL: %s is not a supported lock type" % lock_type)
            return False

        # Waiting for update token
        self.get_update_token()

        if not self._add_queue(host, lock_type):
            _print("FAIL: request_lock() - Could not lock request to queue")
            self.release_update_token()
            return False
        _print("INFO: Added lock request to queue")
        self.release_update_token()
        self.show_queue()
        return True

    def _add_lock(self, host, lock_type, beaker_task_id):
        """
        Set the host
        """

        if not os.path.isdir(self._directory_path):
            if not libsan.host.linux.mkdir(self._directory_path):
                _print("FAIL: _add_lock() - Could not create %s" % self._directory_path)
                return False

        if not host or not lock_type or not beaker_task_id:
            _print("FAIL: _add_lock() - requires host, lock_type and task_id parameters")
            return False

        if not libsan.host.linux.sync(self._directory_path):
            _print("FAIL: Could not sync %s" % self._directory_path)
            return False

        lock_data = _read_file(self._lock_file_path)
        if not beaker_task_id:
            # TODO
            beaker_task_id = "manual"
        device_info = "%s %s %s %s" % (host, lock_type,
                                       libsan.host.linux.time_stamp(in_seconds=True),
                                       beaker_task_id)
        _print("DEBUG: _add_lock() - Adding %s to lock data:" % device_info)
        print(lock_data)
        if lock_data:
            # In case we got a "exclusive" lock request, but lock data is not empty, we will not
            # add it to lock data
            if lock_type == "exclusive":
                _print("INFO: There is an exclusive lock request on queue, "
                       "but lock data is not empty. Need to wait lock data to get empty first...")
                self.show_lock()
                return False

            for locked_host in lock_data:
                # _print("DEBUG: _add_lock() - Checking if %s match on %s" % (host, locked_host))
                m = self._lock_format_regex.match(locked_host)
                if not m:
                    # TODO
                    _print("WARN: _add_lock() - %s is not valid lock host entry. "
                           "Deleting it..." % locked_host)
                    _delete_item_queue_file(self._lock_file_path, locked_host)
                    continue
                if m.group(1) == host:
                    _print("INFO: %s already locked" % host)
                    return True
                if m.group(2) == "exclusive":
                    _print("INFO: Storage device is locked exclusively. "
                           "Need to wait Lock to get empty first...")
                    return False

        # Open file for write with append
        ret, output = run("echo %s >> %s" % (device_info, self._lock_file_path),
                          return_output=True, verbose=False)
        if ret != 0:
            _print("FAIL: _add_lock() - Could not write to %s" % self._lock_file_path)
            print(output)
            return False

        if not libsan.host.linux.sync(self._directory_path):
            _print("FAIL: Could not sync %s" % self._directory_path)
            return False
        _print("DEBUG: _add_lock() - Added %s to lock data" % device_info)
        return True

    def check_lock(self):
        """
        Check which servers are using the device
        """

        _print("INFO: Checking lock status")
        if not os.path.isdir(self._directory_path):
            # It seems no lock has ever been queued to this device
            return None

        # Waiting for update token
        self.get_update_token()

        if not os.path.isfile(self._lock_file_path):
            _print("DEBUG: %s does not exist" % self._lock_file_path)
            # just update lock file, next check will get the info updated
            self._dequeue()
            self.release_update_token()
            return None

        lock_devices = _read_file(self._lock_file_path)
        if not lock_devices:
            _print("DEBUG: check_lock() - there is no lock for %s" % self._device_name)
            # just update lock file, next check will get the info updated
            self._dequeue()
            self.release_update_token()
            return None

        # _print("DEBUG: check_lock() - servers using %s are:" % self._device_name)
        # Check if locker was done by beaker task
        # if so, make sure task is still running
        for locked_host in lock_devices:
            m = self._lock_format_regex.match(locked_host)
            if not m:
                # invalid host format, delete it
                _print("WARN: %s is not valid lock host entry. Deleting it..." % locked_host)
                _delete_item_queue_file(self._lock_file_path, locked_host)
                continue

            # Check if lock was set by manual test run or beaker job
            if m.group(4) == "manual":
                current_timestamp = int(libsan.host.linux.time_stamp(in_seconds=True))
                timeout = int(m.group(3)) + (60 * 60)
                # timeout if locked manually for more than 1 hour
                if current_timestamp > timeout:
                    _print("INFO: manual lock '%s' timeout! Deleting lock..." % locked_host)
                    _delete_item_queue_file(self._lock_file_path, locked_host)
                    lock_devices.remove(locked_host)
                else:
                    _print("INFO: found manual lock, still has %s min left before timeout"
                           % ((timeout - current_timestamp) / 60))
                continue

            # Check if beaker task is still running
            t_status = beaker.get_task_status(m.group(4))
            if not t_status:
                # for some reason could not get beaker job status
                _print("WARN: Could not get status of beaker task %s" % m.group(4))
                # Check if it has been running for too long, in that case we remove the lock
                current_timestamp = int(libsan.host.linux.time_stamp(in_seconds=True))
                # 4hrs before timeout
                timeout = int(m.group(3)) + (4 * 60 * 60)
                # timeout if locked manually for more than 1 hour
                if current_timestamp > timeout:
                    _print("INFO: Task %s lock '%s' timeout! Deleting lock..." %
                           (m.group(4), locked_host))
                    _delete_item_queue_file(self._lock_file_path, locked_host)

            if t_status and t_status != "Running":
                # For example forgot to release the lock or server crashed...
                _print("INFO: Task for %s is on %s status. Not valid lock. "
                       "Deleting it..." % (locked_host, t_status))
                _delete_item_queue_file(self._lock_file_path, locked_host)
                # it will just be updated when we run check again

        # refresh queue, new info will get when we run check again
        self._dequeue()

        self.release_update_token()
        self.show_lock()
        return lock_devices

    def get_my_lock_info(self):
        """
        Check info for the my own lock
        """
        host = libsan.host.linux.hostname()
        self.check_lock()
        locked_list = self.check_lock()
        if locked_list:
            for locked_host in locked_list:
                m = self._lock_format_regex.match(locked_host)
                if m:
                    if host == m.group(1):
                        return locked_host
        return None

    def get_lock(self):
        """
        Wait until get the lock
        """
        host = libsan.host.linux.hostname()
        while True:
            task_id = beaker.get_task_id()
            # if it is a beaker task running
            if task_id:
                timeout = beaker.get_task_timeout(task_id)
                if timeout and timeout < 120:
                    _print("FAIL: Beaker task is about to time out, do not wait for lock")
                    _delete_item_queue_file(self._queue_file_path, host)
                    return None
            locked_list = self.check_lock()
            if locked_list:
                for locked_host in locked_list:
                    m = self._lock_format_regex.match(locked_host)
                    if m:
                        if host == m.group(1):
                            # we got the lock
                            return locked_host
            self.show_queue()
            self.show_lock()
            libsan.host.linux.sleep(60)

    def release_lock(self, lock_type, wait_time=120):
        """
        Release the lock
        """
        if not lock_type:
            _print("FAIL: release_lock() - requires lock_type parameter")
            return False

        host = libsan.host.linux.hostname()

        # Waiting for update token
        self.get_update_token()

        lock_devices = _read_file(self._lock_file_path)
        if not lock_devices:
            _print("INFO: release_lock() - Empty lock data")
            self.release_update_token()
            return True
        for locked_host in lock_devices:
            m = self._lock_format_regex.match(locked_host)
            if not m:
                # invalid host format, delete it
                _print("WARN: release_lock() - %s is not valid lock host entry. "
                       "Deleting it..." % locked_host)
                _delete_item_queue_file(self._lock_file_path, locked_host)
                continue
            if host == m.group(1):
                _print("INFO: Releasing lock %s" % locked_host)
                _delete_item_queue_file(self._lock_file_path, locked_host)

        self.release_update_token()
        if self._monitor_pid:
            libsan.host.linux.kill_pid(self._monitor_pid)
        # slepp 5mins to allow other servers to request lock
        _print("INFO: lock from %s is released. Waiting %s seconds to give chance "
               "to other servers" % (self._lock_file_path, wait_time))
        libsan.host.linux.sleep(wait_time)
        return True

    # def monitor_lock(self):
    #   """
    #   """
    #   host = linux.hostname()
    #   while True:
    #       locked_host = self.get_my_lock_info(wait_for_lock=False)
    #       if locked_host:
    #           m = self._lock_format_regex.match(locked_host)
    #           if m and m.group(1) == host:
    #               _print("Refreshing lock info...")
    #               _delete_item_queue_file(self._lock_file_path, locked_host)
    #               self._add_host(m.group(1), m.group(2))
    #       linux.sleep(5*60)

    #   _print("INFO: Stopping monitoring lock")
    #   return False

    def show_lock(self):
        """
        Show hosts with lock
        """
        _print("INFO: ### Showing NFS lock data #########")
        cmd = "cat %s " % self._lock_file_path
        ret, output = run(cmd, return_output=True, verbose=False)
        print(output)
        _print("INFO: ### End NFS lock data #########")
        return True

    #   ######### TEST ###############
    def _self_test(self):
        """
        Basic self test
        """
        # device_name = "self_test"
        self.mount_lock()
        _print("######## TEST 1 #########")
        self.get_update_token()
        self.release_update_token()
        _print("#PASS - TEST 1")
        _print("######## TEST 2 #########")
        self.request_lock("shared")
        _print("#TEST2: Requested shared lock")
        self.check_lock()
        _print("#TEST2: checked lock")
        self.show_queue()
        _print("#TEST2: Trying to get lock")
        self.get_lock()
        _print("#TEST2: Got lock")
        self.show_lock()
        _print("#TEST2: Trying to release lock")
        self.release_lock("shared")
        _print("#TEST2: Lock released")
        self.show_lock()
        _print("#TEST2: Released shared lock")
        _print("#PASS - TEST 2")
        _print("######## TEST 3 #########")
        self.show_queue()
        self.show_lock()
        _print("#PASS - TEST 3")
        self.umount_lock()

        # linux.umount(self._mount_point)
        # if not self.mount_lock():
        #   _print("FAIL: _self_test() - Could not mount NFS share")
        #   linux.umount(self._mount_point)
        #   return False

        # self.show_lock()

        # if not self.request_lock("shared"):
        #   _print("FAIL: _self_test() - Could not request lock")
        #   linux.umount(self._mount_point)
        #   return False
        # self.show_queue()

        # self.show_lock()

        # if not self._add_queue(host2, "shared"):
        #   _print("FAIL: _self_test() - Could not add %s to queue" % host2)
        #   linux.umount(self._mount_point)
        #   return False

        # self.show_lock()

        # if not self.check_lock():
        #   _print("INFO: _self_test() - No server pending on %s" % self._device_name)

        # if not self.check_lock():
        #   _print("FAIL: _self_test() - We expected to find a device waiting for lock")
        #   linux.umount(self._mount_point)
        #   return False

        # if not self.release_lock("shared"):
        #   _print("FAIL: Could not release lock")
        #   linux.umount(self._mount_point)
        #   return False

        # if not self.check_lock():
        #   _print("INFO: _self_test() - No server pending on %s" % self._device_name)
        # self.show_lock()

        # linux.umount(self._mount_point)
        return True



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import os
from stqe.host.atomic_run import parse_ret

targetd_cfg = "/etc/target/targetd.yaml"


def prepare_conf():
    errors = []
    do_nothing = True

    if not os.path.isfile(targetd_cfg):
        print("FAIL: %s does not exist" % targetd_cfg)
        errors.append("FAIL: %s does not exist" % targetd_cfg)
        return errors

    with open(targetd_cfg, "r") as read_config:
        lines = read_config.readlines()
    if lines[0].startswith("# modified"):
        return True
    for line in lines:
        if line.startswith("# defaults below"):
            do_nothing = False
            continue
        if do_nothing or line.startswith("# log level"):
            continue
        if line.startswith("# "):
            lines[lines.index(line)] = line.lstrip("# ")
            continue
        lines[lines.index(line)] = line.lstrip("#")
    lines.insert(0, "# Modified by prepare_conf.py\n")
    with open(targetd_cfg, "w") as write_config:
        write_config.writelines(lines)

    return errors


if __name__ == "__main__":
    errs = prepare_conf()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import vg_create, vg_remove, pv_remove
import libsan.host.dm as dm
import libsan.host.linux as linux
import stqe.host.tc
import sys
import argparse
import traceback

TestObj = None

loop_dev = None

vg_name = "vgtest"
pool_name = "test_pool"
thin1_name = "thin1"
thin2_name = "thin2"
mnt_point_thin1 = "/mnt/thin1"
mnt_point_thin2 = "/mnt/thin2"


def start_test(filesystem, errwhenfull):
    global TestObj

    print(80 * "#")
    print("INFO: Starting test on FS %s and errwhenfull=%s" % (filesystem, errwhenfull))
    print(80 * "#")

    _clean_up()

    # create loop device of 1G
    global loop_dev
    loop_dev = create_loopdev(size=1024)
    if not loop_dev:
        TestObj.tfail("Could not create loop device")
        return False

    if not vg_create(vg_name, loop_dev):
        TestObj.tfail("Could not create VG \"%s\"" % vg_name)
        return False

    print("INFO: Creating LVs")
    if run("lvcreate --thinpool %s -L 800M %s --errorwhenfull %s" % (pool_name, vg_name, errwhenfull)) != 0:
        TestObj.tfail("Could not create %s [%s - errorwhenfull=%s]" % (pool_name, filesystem, errwhenfull))
        return False
    if run("lvcreate -V 1G -T %s/%s -n %s" % (vg_name, pool_name, thin1_name)) != 0:
        TestObj.tfail("Could not create %s [%s - errorwhenfull=%s]" % (thin1_name, filesystem, errwhenfull))
        return False
    if run("lvcreate -V 1G -T %s/%s -n %s" % (vg_name, pool_name, thin2_name)) != 0:
        TestObj.tfail("Could not create %s [%s - errorwhenfull=%s]" % (thin2_name, filesystem, errwhenfull))
        return False

    print("INFO: Displaying LVs")
    run("lvs -a -o +devices")

    dm.dm_show_table()

    thin1_device = "/dev/mapper/%s-%s" % (vg_name, thin1_name)
    print("INFO:Going to create FS(%s) on %s" % (filesystem, thin1_device))
    if run("mkfs.%s %s" % (filesystem, thin1_device)) != 0:
        TestObj.tfail("Could not create FS(%s) on %s with errorwhenfull=%s" % (filesystem, thin1_device, errwhenfull))
        return False

    if not linux.mkdir(mnt_point_thin1):
        TestObj.tfail(
            "Could not create directory %s - FS(%s) with errorwhenfull=%s" % (mnt_point_thin1, filesystem, errwhenfull))
        return False

    if not linux.mount(thin1_device, mnt_point_thin1):
        TestObj.tfail("Could not mount %s - FS(%s) with errorwhenfull=%s" % (mnt_point_thin1, filesystem, errwhenfull))
        return False

    thin2_device = "/dev/mapper/%s-%s" % (vg_name, thin2_name)
    print("INFO:Going to create FS(%s) on %s" % (filesystem, thin2_device))
    if run("mkfs.%s %s" % (filesystem, thin2_device)) != 0:
        TestObj.tfail("Could not create FS(%s) on %s with errorwhenfull=%s" % (filesystem, thin2_device, errwhenfull))
        return False

    if not linux.mkdir(mnt_point_thin2):
        TestObj.tfail(
            "Could not create directory %s- FS(%s) with errorwhenfull=%s" % (mnt_point_thin2, filesystem, errwhenfull))
        return False

    if not linux.mount(thin2_device, mnt_point_thin2):
        TestObj.tfail("Could not mount %s- FS(%s) with errorwhenfull=%s" % (mnt_point_thin2, filesystem, errwhenfull))
        return False

    run("lvs -a")

    print("INFO: Going create file on %s" % mnt_point_thin1)
    filename = "file1.img"
    # For dd to sync data
    if run("dd conv=fdatasync if=/dev/urandom of=%s/%s bs=1M count=500" % (mnt_point_thin1, filename)) != 0:
        TestObj.tfail(
            "Could not create file on %s - FS(%s) with errorwhenfull=%s" % (mnt_point_thin1, filesystem, errwhenfull))
        return False

    linux.sync()
    run("lvs -a")

    # if not TestObj.tnok("cp %s/%s %s/"% (mnt_point_thin1, filename, mnt_point_thin2)):
    if run("dd conv=fdatasync if=/dev/urandom of=%s/%s bs=1M count=500" % (mnt_point_thin2, filename)) == 0:
        TestObj.tfail("Should be possible to create file on %s - FS(%s) with errorwhenfull=%s" % (
            mnt_point_thin2, filesystem, errwhenfull))
        run("lvs -a")
        dm.dm_show_status()
        run("sync")
        run("ls -allh %s/" % mnt_point_thin2)
        run("tail /var/log/messages")
        return False

    linux.sync()
    run("lvs -a")

    print(80 * "#")
    TestObj.tpass("PASS: Test on FS %s and errwhenfull=%s" % (filesystem, errwhenfull))
    print(80 * "#")

    return True


def _clean_up():
    global vg_name, loop_dev

    linux.umount(mnt_point_thin1)
    linux.umount(mnt_point_thin2)

    # make sure any failed device is removed
    run("dmsetup remove_all")
    if not vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        if not pv_remove(loop_dev):
            TestObj.tfail("Could not delete PV \"%s\"" % loop_dev)
        linux.sleep(1)
        if not delete_loopdev(loop_dev):
            TestObj.tfail("Could not remove loop device %s" % loop_dev)


def main():
    global TestObj

    parser = argparse.ArgumentParser()
    parser.add_argument("--filesystem", "-f", required=False, dest="fs",
                        help="Filesystem name", metavar="filesytem")

    args = parser.parse_args()

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    # test if IO fails when error_if_no_space and also if queue IO times out
    errwhenfull = ["y", "n"]

    filesystem = linux.get_default_fs()

    if args.fs:
        filesystem = args.fs

    for err in errwhenfull:
        try:
            start_test(filesystem, err)
        except Exception as e:
            print(e)
            traceback.print_exc()
            e = sys.exc_info()[0]
            TestObj.tfail("There was some problem while running the test (%s)" % e)
            print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import os
from stqe.host.atomic_run import parse_ret, atomic_run
from stqe.host.persistent_vars import read_var, clean_var
from libsan.host.mp import mp_start_service, flush_all, mp_service_name
from libsan.host.linux import is_installed, is_service_running, \
    service_start, service_status, dist_ver


def start_multipath():
    errors = []
    multipath = None
    mp_socket_process = "multipathd.socket"
    if os.path.isfile("/tmp/START_MULTIPATH"):
        multipath = read_var("START_MULTIPATH")

    if not is_installed('device-mapper-multipath'):
        print("WARN: Skipping setup! Package device-mapper-multipath is not installed!")
        return errors

    if multipath is not None and multipath == 1:
        print("WARN: Skipping start of the services because they have not been running!")
        atomic_run("Cleaning var START_MULTIPATH",
                   command=clean_var,
                   var="START_MULTIPATH",
                   errors=errors)
        
        return errors

    if float(dist_ver()) >= 8.0 and not is_service_running(mp_socket_process):
        ret = atomic_run("Starting service %s" % mp_socket_process,
                         service_name=mp_socket_process,
                         command=service_start,
                         errors=errors
                         )
        if ret != 5:
            if not is_service_running(mp_socket_process):
                msg = "FAIL: Service %s is not running" % mp_socket_process
                print(msg)
                errors.append(msg)
                return errors

    atomic_run("Starting service multipathd",
               command=mp_start_service,
               errors=errors
               )

    atomic_run("Flushing all",
               command=flush_all,
               errors=errors
               )

    if not is_service_running(mp_service_name()) and os.path.isfile("/etc/multipath.conf"):
        print("Printing %s status" % mp_service_name())
        service_status(mp_service_name())

        msg = "FAIL: Service %s is not running!" % mp_service_name()
        print(msg)
        errors.append(msg)
        return errors

    return errors


if __name__ == "__main__":
    errs = start_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from os import environ


def create_volume():
    print("INFO: Creating volume.")
    errors = []

    id = ""
    if 'fmf_id' in environ:
        id = "_" + environ['fmf_id']

    vol_name = read_var("LSM_VOL_NAME" + id)
    vol_size = environ['fmf_vol_size' + id]
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    ret, data = atomic_run("Creating volume %s" % vol_name,
                           command=lsm.volume_create,
                           name=vol_name,
                           size=vol_size,
                           pool=pool_id,
                           return_output=True,
                           errors=errors)

    for line in data.splitlines():
        if vol_name in line:
            vol_id = line.split()[0].strip()
            atomic_run("Writing var LSM_VOL_ID",
                       command=write_var,
                       var={"LSM_VOL_ID" + id: vol_id},
                       errors=errors)

    atomic_run("Writing var LSM_VOL_SIZE" + id,
               command=write_var,
               var={"LSM_VOL_SIZE" + id: vol_size},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_volume()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, read_env, write_var
import time


def create_pool():
    errors = []
    key_desc = ''
    trust_url = None
    clevis = None
    tang_url = None
    thumbprint = None
    no_overprovision = None
    stratis = Stratis()

    pool_name = read_env('fmf_pool_name')
    id = ""
    try:
        id = "_" + str(read_env("fmf_pool_id"))
        pool_name = pool_name + id
    except KeyError:
        pass
    try:
        key_desc = read_var(read_env('fmf_key_desc'))
    except KeyError:
        pass
    try:
        trust_url = read_env("fmf_trust_url")
    except KeyError:
        pass
    try:
        thumbprint = read_var(read_env("fmf_thumbprint"))
    except KeyError:
        pass
    try:
        clevis = read_env("fmf_clevis")
    except KeyError:
        pass
    try:
        tang_url = read_var(read_env("fmf_tang_url"))
    except KeyError:
        pass
    try:
        no_overprovision = read_env("fmf_no_overprovision")
    except KeyError:
        pass

    if thumbprint and trust_url:
        trust_url = None

    previous = ""
    try:
        previous = int(id[1:]) - 1
        if previous == 1:
            previous = ""
        else:
            previous = "_" + str(previous)
    except ValueError:
        pass
    if previous:
        print("previous is %s, id is %s," % (previous, id))
    blockdevs = read_var("STRATIS_FREE")
    if blockdevs is None:
        blockdevs = read_var("STRATIS_DEVICE")

    if "single" in read_env("fmf_name"):
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        blockdevs = [blockdev for blockdev in blockdevs if blockdev]
        free = blockdevs[:]
        blockdevs = free.pop()
        atomic_run("Writing var STRATIS_FREE",
                   command=write_var,
                   var={'STRATIS_FREE': free},
                   errors=errors)
        atomic_run("Writing var STRATIS_DEVICE",
                   command=write_var,
                   var={'STRATIS_DEVICE': blockdevs},
                   errors=errors)

    time.sleep(2)
    atomic_run(message="Triggering udev",
               command=run,
               cmd="udevadm trigger; udevadm settle",
               errors=errors)
    if clevis or key_desc:
        atomic_run("Creating stratis pool %s." % pool_name,
                   command=stratis.pool_create,
                   pool_name=pool_name,
                   blockdevs=blockdevs,
                   key_desc=key_desc,
                   clevis=clevis,
                   thumbprint=thumbprint,
                   tang_url=tang_url,
                   trust_url=trust_url,
                   no_overprovision=no_overprovision,
                   errors=errors)
    else:
        atomic_run("Creating stratis pool %s." % pool_name,
                   command=stratis.pool_create,
                   pool_name=pool_name,
                   no_overprovision=no_overprovision,
                   blockdevs=blockdevs,
                   errors=errors)

    atomic_run("Writing var STRATIS_POOL%s" % id,
               command=write_var,
               var={'STRATIS_POOL%s' % id: pool_name},
               errors=errors)

    pool_uuid = stratis.get_pool_uuid(pool_name)

    atomic_run("Writing var STRATIS_POOL_UUID",
               command=write_var,
               var={'STRATIS_POOL_UUID': pool_uuid},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_pool()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.persistent_vars import write_var, read_var
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI
from os import environ


def remove_nones(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


def create_iscsi():
    errors = []
    target_iqn = None
    tpg = None
    args = dict(storage_object=None, lun=None)
    args["storage_object"] = "/backstores/user:qcow/" + read_var("USER:QCOW_NAME")
    try:
        args["lun"] = environ["fmf_lun_id"]
    except KeyError:
        pass
    target = TargetCLI(path="/iscsi")

    ret, data = atomic_run("Creating iscsi",
                           return_output=True,
                           command=target.create,
                           errors=errors
                           )

    if ret != 0:
        print("FAIL: Could not create iscsi target !")
        errors.append("FAIL: Could not create iscsi target !")
        return errors

    lines = data.splitlines()
    for line in lines:
        if "iqn" in line:
            target_iqn = line.split()[2][:-1]
            continue
        if "TPG" in line:
            tpg = "tpg" + line.split()[2][:-1]
            continue

    target.path = "/iscsi/" + target_iqn + "/" + tpg + "/" + "luns"

    atomic_run("Creating lun with storage object: %s" % args["storage_object"],
               command=target.create,
               errors=errors,
               **remove_nones(args)
               )

    target.path = "/iscsi/" + target_iqn + "/" + tpg

    atomic_run("Setting parameter generate_node_acls to '1'",
               generate_node_acls=1,
               group="attribute",
               command=target.set,
               errors=errors
               )

    atomic_run("Setting parameter demo_mode_write_protect to '0'",
               demo_mode_write_protect=0,
               group="attribute",
               command=target.set,
               errors=errors
               )

    atomic_run("Writing var TPG",
               command=write_var,
               var={"TPG": tpg},
               errors=errors)

    atomic_run("Writing var TARGET IQN",
               command=write_var,
               var={"TARGET_IQN": target_iqn},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_iscsi()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.linux as linux

TestObj = None


def _lvm_test():
    # Check if thin and thin-pool are supported segment types on lvm
    TestObj.tok("lvm segtypes | grep -w \"thin$\"")
    TestObj.tok("lvm segtypes | grep -w \"thin-pool$\"")


def start_test():
    global TestObj

    print(80 * "#")
    print("INFO: Starting lvm Thin Provisioning Misc test")
    print(80 * "#")

    _lvm_test()

    return True


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.fcoe import setup_soft_fcoe
from libsan.sanmgmt import choose_mpaths
from stqe.host.persistent_vars import write_var


def setup_fcoe():
    print("INFO: Setting up FCoE.")
    errors = []

    atomic_run("Setting up soft FCoE",
               command=setup_soft_fcoe,
               errors=errors)

    mpaths = atomic_run(message="Getting multipath information",
                        command=choose_mpaths,
                        errors=errors)

    disk = None
    for mpath in mpaths:
        if 'FCoE' in mpaths[mpath]['transport_types']:
            disk = mpaths[mpath]['disk'].keys().pop()

    if not disk:
        msg = "FAIL: Could not find FCoE disk."
        print(msg)
        errors += msg
        return errors

    atomic_run("Cleaning superblock of device /dev/%s" % disk,
               command=run,
               cmd="dd if=/dev/zero of=/dev/%s bs=4k count=2" % disk,
               errors=errors)

    atomic_run("Writing var VDO_DEVICE",
               command=write_var,
               var={'VDO_DEVICE': "/dev/%s" % disk},
               errors=errors)

    atomic_run("Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_fcoe()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

# Regression test for:
# BZ1311818 - software iSCSI: Data corruption occured with 4K I/O is sent repeatedly during ethernet port failure tests

from __future__ import absolute_import, division, print_function, unicode_literals
import libsan.host.dt as dt
import libsan.host.linux as linux
import libsan.host.mp as mp
import libsan.host.fcoe as fcoe
import libsan.sanmgmt as sanmgmt
import stqe.host.tc
import sys
import os
import traceback
import argparse

TestObj = None

runtime = "1h"

test_ifaces = []
obj_sanmgmt = None


def start_test():
    global TestObj, obj_sanmgmt, test_ifaces, runtime

    print(80 * "#")
    print("INFO: Starting alternate interface failover test")
    print(80 * "#")

    _clean_up()

    choosen_devs = sanmgmt.choose_mpaths()
    if not choosen_devs:
        TestObj.tfail("Could not find any device to perform IO")
        return False

    for dev_name in choosen_devs.keys():
        test_device = "/dev/mapper/%s" % dev_name
        print("INFO: Starting IO test on %s" % test_device)
        mp_info = mp.multipath_query_all(dev_name)
        if not mp_info:
            TestObj.tfail("Could not query info for %s" % dev_name)
            return False

        obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(dev_name)
        if not obj_sanmgmt:
            TestObj.tfail("Could not create SanMgmt obj")
            return False

        test_ifaces = []
        # If it list is empty nothing will be added
        test_ifaces.extend(mp_info["h_wwpns"])
        # We use MAC interface for iSCSI devices
        test_ifaces.extend(mp_info["iface_macs"])

        if len(test_ifaces) < 2:
            TestObj.tfail("Mpath device %s has less than 2 host interfaces" % dev_name)
            print(test_ifaces)
            return False

        flag_dt_done = 0
        exit_status = 1
        dt_pid = dt.dt_stress_background(of="/dev/mapper/%s" % dev_name, time=runtime)
        while flag_dt_done == 0:
            for iface in test_ifaces:
                mp.multipath_show()
                if not obj_sanmgmt.link_down(iface):
                    TestObj.tfail("Could not bring %s down" % iface)
                    return False
                print("INFO: Waiting 40s with %s DOWN" % iface)
                linux.sleep(40)
                if not obj_sanmgmt.link_up(iface):
                    TestObj.tfail("Could not bring %s up" % iface)
                    return False
                print("INFO: Waiting 600s with all ifaces UP")
                # wait a bit to allow multipath detect the path is restored
                linux.sleep(600)

                flag_dt_done, exit_status = os.waitpid(dt_pid, os.WNOHANG)
                if flag_dt_done:
                    break

        if exit_status == 0:
            return True

    return False


def _clean_up():
    global obj_sanmgmt, test_ifaces

    if not obj_sanmgmt:
        return True

    # make sure the interfaces will be up when test finishes
    for iface in test_ifaces:
        obj_sanmgmt.link_up(iface)


def main():
    global TestObj, runtime

    parser = argparse.ArgumentParser()
    parser.add_argument("--runtime", "-t", required=False, dest="runtime",
                        help="runtime", metavar="time")

    args = parser.parse_args()
    if args.runtime:
        runtime = args.runtime

    # Environment variable will have preference over parameter value
    if "RUNTIME" in list(os.environ.keys()):
        runtime = os.environ['RUNTIME']

    # Try to enable SW FCoE and iSCSI
    fcoe.setup_soft_fcoe()
    sanmgmt.setup_iscsi()

    TestObj = stqe.host.tc.TestClass()

    try:
        start_test()
    except Exception as e:
        print(e)
        # traceback.print_exc()
        # e = sys.exc_info()[0]
        TestObj.tfail("FAIL: Exception when running test (%s)" % traceback.format_exc())

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

#
# Bugs related:
#

from __future__ import absolute_import, division, print_function, unicode_literals
import traceback
import sys
import stqe.host.tc
from libsan.host.loopdev import create_loopdev, delete_loopdev
import libsan.host.lvm as lvm
import libsan.host.linux as linux

TestObj = None

loop_dev = None

vg_name = "VG"
pool_name = "POOL"
origin_mnt = "/tmp/origin_mnt"
snap_mnt = "/tmp/snap_mnt"


def main():
    global TestObj

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        print(e)
        traceback.print_exc()
        e = sys.exc_info()[0]
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


def start_test():
    global TestObj

    TestObj = stqe.host.tc.TestClass()
    TestObj.trun("rpm -q lvm2")

    _clean_up()

    global loop_dev
    loop_dev = create_loopdev("loop0", 512)
    if not loop_dev:
        TestObj.tfail("Could not create loop device")
        return False

    if not lvm.vg_create(vg_name, loop_dev):
        TestObj.tfail("Could not create VG \"%s\"" % vg_name)
        return False

    print("INFO: Creating LV")
    TestObj.tok("lvcreate -V100m -l125 -T %s/%s -n origin"
                % (vg_name, pool_name))
    TestObj.tok("lvchange --discards nopassdown %s/%s" % (vg_name, pool_name))
    if not linux.mkfs("/dev/%s/origin" % vg_name, "ext4"):
        TestObj.tfail("Could not create FS for origin")
        return False
    if not linux.mkdir(origin_mnt):
        TestObj.tfail("Could not create mnt directory to origin")
        return False
    if not TestObj.tok("mount /dev/%s/origin %s" % (vg_name, origin_mnt)):
        return False

    # Create a file
    if not TestObj.tok("sync; dd if=/dev/zero of=%s/file1 bs=1M count=60; sync" % origin_mnt):
        return False

    print("INFO: Creating snapshot")
    # create the snapshot
    if not TestObj.tok("lvcreate -K -s %s/origin -n %s/snap1" % (vg_name, vg_name)):
        return False
    linux.sleep(5)

    # Show VG discard info
    TestObj.trun("lvs %s -o+discards" % vg_name)
    pool_info = lvm.lv_info(pool_name, vg_name, "data_percent")
    origin_info = lvm.lv_info("origin", vg_name, "data_percent")
    snap_info = lvm.lv_info("snap1", vg_name, "data_percent")
    pre_dp_pool = float(pool_info["data_percent"])
    pre_dp_origin = float(origin_info["data_percent"])
    pre_dp_snap = float(snap_info["data_percent"])

    # rm files on the origin and do discard
    TestObj.tok("rm -rf %s/file1;sync" % origin_mnt)
    TestObj.tok("fstrim -vvv %s;sync" % origin_mnt)
    TestObj.trun("umount %s" % origin_mnt)
    linux.sleep(5)

    print("INFO: Checking if data percentage got reduced only on origin")
    TestObj.trun("lvs %s -o+discards" % vg_name)
    pool_info = lvm.lv_info(pool_name, vg_name, "data_percent")
    origin_info = lvm.lv_info("origin", vg_name, "data_percent")
    snap_info = lvm.lv_info("snap1", vg_name, "data_percent")
    now_dp_pool = float(pool_info["data_percent"])
    now_dp_origin = float(origin_info["data_percent"])
    now_dp_snap = float(snap_info["data_percent"])

    pool_diff = pre_dp_pool / now_dp_pool

    # the pool's data percentage is a little different from the previous one (2% tolerance)
    if pool_diff > 1.02 or pool_diff < 0.98:
        TestObj.tfail("Data pool different is too big: %s" % pool_diff)
        print("FAIL: pre_dp_pool: %s - now_dp_pool: %s - diff: %s" % (pre_dp_pool, now_dp_pool, pool_diff))
        return False

    # the data percentage of snapshot, it should keep the same
    if now_dp_snap != pre_dp_snap:
        TestObj.tfail("data percentage of snap should not have changed")
        print("FAIL: pre_dp_snap: %s - now_dp_snap: %s" % (pre_dp_snap, now_dp_snap))
        return False

    # the data percentage of origin, it should reduce
    if now_dp_origin >= pre_dp_origin:
        TestObj.tfail("data percentage of origin should have reduced")
        print("FAIL: pre_dp_origin: %s - now_dp_origin: %s" % (pre_dp_origin, now_dp_origin))
        return False

    #
    # do discards on the snap1
    #
    print("INFO: Deleting data from snapshot")
    if not linux.mkdir(snap_mnt):
        TestObj.tfail("Could not create mnt directory to snap")
        return False
    if not TestObj.tok("mount /dev/%s/snap1 %s" % (vg_name, snap_mnt)):
        return False
    TestObj.tok("rm -rf %s/file1;sync" % snap_mnt)
    TestObj.tok("fstrim -vvv %s;sync" % snap_mnt)
    linux.sleep(5)

    print("INFO: Checking if data percentage got reduced on snapshot and pool")
    TestObj.trun("lvs %s -o+discards" % vg_name)
    pool_info = lvm.lv_info(pool_name, vg_name, "data_percent")
    origin_info = lvm.lv_info("origin", vg_name, "data_percent")
    snap_info = lvm.lv_info("snap1", vg_name, "data_percent")
    post_dp_pool = float(pool_info["data_percent"])
    post_dp_origin = float(origin_info["data_percent"])
    post_dp_snap = float(snap_info["data_percent"])

    # the pool's data percentage should be reduced
    if post_dp_pool >= now_dp_pool:
        TestObj.tfail("Data pool percentage did not reduce")
        print("FAIL: post_dp_pool: %s - now_dp_pool: %s" % (post_dp_pool, now_dp_pool))
        return False

    # the data percentage of snapshot should be reduced
    if post_dp_snap >= now_dp_snap:
        TestObj.tfail("data percentage of snap should have changed")
        print("FAIL: post_dp_snap: %s - now_dp_snap: %s" % (post_dp_snap, now_dp_snap))
        return False

    # the data percentage of origin, it should keep the same
    if post_dp_origin != now_dp_origin:
        TestObj.tfail("data percentage of origin should not have changed")
        print("FAIL: post_dp_origin: %s - now_dp_origin: %s" % (post_dp_origin, now_dp_origin))
        return False

    TestObj.trun("umount %s" % snap_mnt)

    return True


def _clean_up():
    global vg_name, loop_dev

    TestObj.trun("umount %s" % origin_mnt)
    TestObj.trun("umount %s" % snap_mnt)

    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        if not lvm.pv_remove(loop_dev):
            TestObj.tfail("Could not delete PV \"%s\"" % loop_dev)
        delete_loopdev(loop_dev)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


# fs_snap_delete_success is part of fs_snap_create_success() in this same directory


def fs_snap_delete_fail():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    snap_id = read_var("LSM_FS_SNAP_ID")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail deleting snapshot on FS without any paramethers with protocol %s"
                        % config['protocol'], 'command': lsm.fs_snap_delete},
            {'message': "Trying to fail deleting snapshot on FS without 'fs' with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_delete, 'snap': snap_id},
            {'message': "Trying to fail deleting snapshot on FS without 'snap' with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_delete, 'fs': fs_id},
            {'message': "Trying to fail deleting snapshot on FS by using snap name instead of fs-ID with protocol %s"
                        % config['protocol'], 'command': lsm.fs_snap_delete, 'fs': fs_id, 'snap': snap_name},
            {'message': "Trying to fail deleting snapshot on NONEXISTENT FS with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_delete, 'fs': "NONEXISTENT", 'snap': snap_id},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 2:
        errs = fs_snap_delete_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.stratis import Stratis
from stqe.host.persistent_vars import read_var
from time import sleep


def fs_limit():
    errors = []
    stratis = Stratis()
    number_of_iterations = 100
    pool_name = "pool_create_destroy"
    fs_name = "fs_create_destroy"
    blockdev = read_var("STRATIS_DEVICE")

    for i in range(number_of_iterations):
        print(f"INFO: Iteration number: {i}")
        atomic_run("Creating pool",
                   pool_name=pool_name,
                   blockdevs=blockdev,
                   command=stratis.pool_create,
                   errors=errors
                   )

        sleep(1)

        atomic_run("Destroying pool",
                   pool_name=pool_name,
                   command=stratis.pool_destroy,
                   errors=errors
                   )

    atomic_run("Creating pool",
               pool_name=pool_name,
               blockdevs=blockdev,
               command=stratis.pool_create,
               errors=errors
               )

    for i in range(number_of_iterations):
        print(f"INFO: Iteration number: {i}")
        atomic_run("Creating fs",
                   pool_name=pool_name,
                   fs_name=fs_name,
                   command=stratis.fs_create,
                   errors=errors
                   )

        sleep(1)

        atomic_run("Destroying fs",
                   pool_name=pool_name,
                   fs_name=fs_name,
                   command=stratis.fs_destroy,
                   errors=errors
                   )

    atomic_run("Destroying pool",
               pool_name=pool_name,
               command=stratis.pool_destroy,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = fs_limit()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_read_cache_policy_update_success():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Getting read cache policy of volume %s." % vol,
                               command=lsm.volume_cache_info,
                               vol=vol,
                               return_output=True,
                               script=True,
                               errors=errors)
        old_policy = None
        for line in data.splitlines():
            if "Read Cache Policy" in line:
                old_policy = line.split().pop().strip()[:-1]  # policy is 'disabled' but cli takes 'disable'

        for policy in ["enable", "disable", "Enable", "Disable", "ENABLE", "DISABLE"]:
            atomic_run("Updating policy of volume %s to be %s with protocol %s." % (vol, policy, config['protocol']),
                       command=lsm.volume_read_cache_policy_update,
                       vol=vol,
                       policy=policy,
                       errors=errors)

        atomic_run("Returning policy of volume %s to former state '%s' with protocol %s." %
                   (vol, old_policy, config['protocol']),
                   command=lsm.volume_read_cache_policy_update,
                   vol=vol,
                   policy=old_policy,
                   errors=errors)
    return errors


def volume_read_cache_policy_update_fail():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail changing read policy of volume without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.volume_read_cache_policy_update},
            {'message': "Trying to fail changing read policy of volume with WRONG volume with protocol %s" %
                        config['protocol'], 'vol': "WRONG", 'policy': "enable",
             'command': lsm.volume_read_cache_policy_update},
            {'message': "Trying to fail changing read policy of volume with WRONG policy with protocol %s" %
                        config['protocol'], 'vol': vol, 'policy': "WRONG",
             'command': lsm.volume_read_cache_policy_update}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_read_cache_policy_update_offline():
    errors = []
    # TODO: Check updating policy of offline volume, create test
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_read_cache_policy_update_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_read_cache_policy_update_fail()
    exit(parse_ret(errs))



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.mp as mp
import libsan.host.linux as linux
import libsan.host.fc as fc
import libsan.host.fcoe as fcoe
import libsan.misc.time as time
import libsan.sanmgmt as sanmgmt
import stqe.host.nfs_lock as nfs_lock
import stqe.host.tc
import re
import sys
import os
import subprocess
import argparse
import traceback

TestObj = None

LOCK_TYPE = "shared"
test_tmos = [35, 45, 61]

clean_up_info = {}
lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = "date \"+%Y-%m-%d %H:%M:%S\""
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")

    print("[%s] %s" % (stdout, string))

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """
    """
    global clean_up_info

    if not clean_up_info:
        return True

    # TODO: Bring link back up
    if "obj_sanmgmt" in clean_up_info and "h_wwpn" in clean_up_info:
        clean_up_info["obj_sanmgmt"].link_up(clean_up_info["h_wwpn"])
        # give some time for r_port to be reconnected
        linux.sleep(10)

    # Restore r_port setting back to original values
    r_port = None
    if "r_port" in clean_up_info:
        r_port = clean_up_info["r_port"]
    if r_port and "fast_io_fail_tmo" in clean_up_info:
        print("INFO: Restoring initial fast_io_fail_tmo value")
        fc.set_value_rport_parameter(r_port, "fast_io_fail_tmo",
                                     clean_up_info["fast_io_fail_tmo"])
    if r_port and "dev_loss_tmo" in clean_up_info:
        print("INFO: Restoring initial dev_loss_tmo value")
        fc.set_value_rport_parameter(r_port, "dev_loss_tmo",
                                     clean_up_info["dev_loss_tmo"])
    mp.mp_start_service()

    if "lock_obj" in clean_up_info:
        clean_up_info["lock_obj"].release_lock(LOCK_TYPE)
    _print("INFO: Clean up done.")
    clean_up_info = {}
    return True


def _compare_rport_status(r_port, expected_state):
    r_port_state = fc.get_value_rport_parameter(r_port, "port_state")
    if not r_port_state:
        print("FAIL: Could not get r_port %s state" % r_port)
        return False
    if r_port_state != expected_state:
        print("FAIL: expected r_port to be with state %s, but it is %s" % (expected_state, r_port_state))
        return False
    return True


def _direct_io_tmo(scsi_disk):
    """
    Perform direct IO to device and return the time it took to fail
    If IO does not fail it returns None
    """
    cmd = "dd if=/dev/%s " % scsi_disk
    cmd += "of=/dev/null count=1 bs=512 iflag=direct"
    io_start_time = time.get_time(in_seconds=True)
    ret, output = run(cmd, return_output=True)
    io_end_time = time.get_time(in_seconds=True)
    if ret == 0:
        # IO did not fail
        return None
    return io_end_time - io_start_time


def dev_loss_tmo_test(mpath_name, tmo, max_link_tmo):
    global clean_up_info, TestObj

    print("INFO: Starting test run for sys/dev_loss_tmo tmo = %s" % tmo)
    if "obj_sanmgmt" not in list(clean_up_info.keys()):
        print("FAIL: Could not get obj_sanmgmt")
        return False

    obj_sanmgmt = clean_up_info["obj_sanmgmt"]

    # Force some IO to make sure there is an active path
    run("dd if=/dev/zero of=/dev/mapper/%s count=512" % mpath_name)

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    mp.multipath_show(mpath_name)
    active_disks = mp.mpath_get_active_disk(mpath_name)
    if not active_disks:
        print("FAIL: Could not find the active path from %s" % mpath_name)
        return False
    # We will test only one of active disks, most common mpath config just have 1 active disk each time anyway
    active_disk = active_disks[0]

    # Stop only after getting the active disk
    print("INFO: Stoping multipathd as it can also control dev_loss_tmo")
    if not mp.mp_stop_service():
        print("FAIL: Could not stop multipath service")
        return False

    clean_up_info["active_disk"] = active_disk

    required_mp_info = ["wwid", "disk"]
    for req in required_mp_info:
        if req not in mp_info:
            print("FAIL: mpath info does not provide %s that is required" % req)
            return False
    wwid = mp_info["wwid"]
    if active_disk not in mp_info["disk"]:
        print("FAIL: Could not find %s on mpath %s" % (active_disk, mpath_name))
        return False
    disk_info = mp_info["disk"][active_disk]

    required_disk_info = ["h_wwpn", "t_wwpn"]
    for req in required_disk_info:
        if req not in disk_info:
            print("FAIL: disk does not provide %s that is required" % req)
            return False
    h_wwpn = disk_info["h_wwpn"]
    t_wwpn = disk_info["t_wwpn"]
    r_port = fc.rport_of_h_wwpn_t_wwpn(h_wwpn=h_wwpn, t_wwpn=t_wwpn)
    if not r_port:
        print("FAIL: Could not find r_port_id")
        return False

    print("INFO:  Choose SCSI disk from mpath: %s" % mpath_name)
    print("\t\tHost WWPN  : %s" % h_wwpn)
    print("\t\tTarget WWPN: %s" % t_wwpn)
    print("\t\tWWID       : %s" % wwid)
    print("\t\tSCSI disk  : %s" % active_disk)
    print("\t\tFC rport   : %s" % r_port)

    # Save h_wwpn to bring it up on clean up
    clean_up_info["h_wwpn"] = h_wwpn
    clean_up_info["r_port"] = r_port

    dev_loss_tmo = fc.get_value_rport_parameter(r_port, "dev_loss_tmo")
    if not dev_loss_tmo:
        print("FAIL: Could not get dev_loss_tmo for %s" % r_port)
        return False
    fast_io_fail_tmo = fc.get_value_rport_parameter(r_port, "fast_io_fail_tmo")
    if not fast_io_fail_tmo:
        print("FAIL: Could not get fast_io_fail_tmo for %s" % r_port)
        return False
    clean_up_info["dev_loss_tmo"] = dev_loss_tmo
    clean_up_info["fast_io_fail_tmo"] = fast_io_fail_tmo
    print("INFO: Original dev_loss_tmo is %s" % dev_loss_tmo)
    print("INFO: Original fast_io_fail_tmo is %s" % fast_io_fail_tmo)

    if not fc.set_value_rport_parameter(r_port, "fast_io_fail_tmo", "off"):
        print("FAIL: Could not set fast_io_fail_tmo of %s to %s" % (r_port, tmo))
        return False
    print("INFO: Active SCSI disk %s(%s) fast_io_fail_tmo is set to 'off'" % (active_disk, r_port))

    if not fc.set_value_rport_parameter(r_port, "dev_loss_tmo", tmo):
        print("FAIL: Could not et dev_loss_tmo of %s to %s" % (r_port, tmo))
        return False
    print("INFO: Active SCSI disk %s(%s) dev_loss_tmo is set to '%s'" % (active_disk, r_port, tmo))

    if not _compare_rport_status(r_port, "Online"):
        return False

    # Bring host port down on switch
    if not obj_sanmgmt.link_down(h_wwpn):
        print("FAIL: Could not bring switch port down")
        return False

    if not _compare_rport_status(r_port, "Blocked"):
        return False

    print("INFO: Checking direct io timeout on failed path")
    io_tmo = _direct_io_tmo(active_disk)
    if io_tmo is None:
        print("FAIL: No I/O error on %s after link failure" % active_disk)
        return False

    if io_tmo > tmo:
        print("FAIL: IO took longer to fail (%ss) than configured (%ss)" % (io_tmo, tmo))
        return False
    print("INFO: I/O successfully failed after %ds" % io_tmo)
    # sleep the remaining timeout
    remaining_time = tmo - io_tmo
    if remaining_time > 0:
        linux.sleep(remaining_time)

    if not _compare_rport_status(r_port, "Not Present"):
        return False

    # Make sure disk device got removed
    linux.wait_udev()
    if os.path.isfile("/dev/%s" % active_disk):
        print("FAIL: SCSI disk should be been removed, but it still exist")
        return False

    # Restore link back up
    if not obj_sanmgmt.link_up(h_wwpn):
        print("FAIL: Could not bring switch port up")
        return False
    # No need to restore port on clean_up
    del clean_up_info["h_wwpn"]

    # Wait port to be restored
    current_port_staus = "Not Present"
    while max_link_tmo > 0:
        print("INFO: Waiting rport came online, timeout: %d" % max_link_tmo)
        linux.sleep(1)
        linux.wait_udev()
        max_link_tmo -= 1
        r_port = fc.rport_of_h_wwpn_t_wwpn(h_wwpn=h_wwpn, t_wwpn=t_wwpn)
        if not r_port:
            # port still not up
            continue
        if not _compare_rport_status(r_port, "Online"):
            # port still not up
            continue
        current_port_staus = "Online"
        new_active_disk = fc.scsi_disk_of_htwwpn_wwid(h_wwpn, t_wwpn, wwid)
        if not new_active_disk:
            continue
        if new_active_disk != active_disk:
            print("INFO: %s got renamed to %s" % (active_disk, new_active_disk))
            active_disk = new_active_disk
        break

    if max_link_tmo == 0 and current_port_staus == "Online":
        print("FAIL: max_link_tmo expired and r_port was not restored")
        return False
    if current_port_staus != "Online":
        # Should not be possible to enter here
        print("FAIL: Did not reach time out, but port still not Online")
        return False

    if _direct_io_tmo(active_disk) is not None:
        print("FAIL: I/O fail, but it should not have to")
        return False
    print("INFO: I/O to %s is restored" % active_disk)
    fc.set_value_rport_parameter(r_port, "fast_io_fail_tmo",
                                 clean_up_info["fast_io_fail_tmo"])
    del clean_up_info["fast_io_fail_tmo"]
    fc.set_value_rport_parameter(r_port, "dev_loss_tmo",

                                 clean_up_info["dev_loss_tmo"])
    del clean_up_info["dev_loss_tmo"]
    mp.mp_start_service()
    linux.sleep(10)
    return True


def execute_test(mpath_name, max_link_tmo):
    global clean_up_info

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        clean_up_info["lock_obj"] = lock_obj
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            return False
        _print("INFO: Good, got NFS lock. Start testing...")

    error = 0
    for tmo in test_tmos:
        if not dev_loss_tmo_test(mpath_name, tmo, max_link_tmo):
            TestObj.tfail("dev_loss_tmo test on %s with timeout %ds" % (mpath_name, tmo))
            break
        print(80 * "#")
        TestObj.tpass("dev_loss_tmo on %s with timeout of %ds" % (mpath_name, tmo))
        print(80 * "#")
    if error:
        return False
    _print("PASS: dev_loss_tmo test")
    return True


def main():
    """
    """
    global TestObj, test_tmos
    PASS = 0
    FAIL = 1

    parser = argparse.ArgumentParser(
        description='dev loss timeout')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to use", metavar="mpath")
    parser.add_argument("--max-link-up-tmo", required=False, dest="max_link_up_tmo", default=20,
                        help="How much time should wait to detect disk", type=int, metavar="number")
    parser.add_argument("--dev-loss-tmo", required=False, dest="dev_loss_tmo", default=None,
                        help="Device loss timeout", type=int, metavar="number")

    args = parser.parse_args()

    TestObj = stqe.host.tc.TestClass()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    if args.dev_loss_tmo:
        test_tmos = [args.dev_loss_tmo]
    max_link_up_tmo = args.max_link_up_tmo

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        # If no mpath is given, make sure soft fcoe is configured
        # Because test will try to find mpath devices
        fcoe.setup_soft_fcoe()

        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    for mpath_name in mpath_name_list:
        try:
            execute_test(mpath_name, max_link_up_tmo)
        except Exception as e:
            print(e)
            TestObj.tfail("Could not execute test on %s" % mpath_name)
            traceback.print_exc()
        _clean_up()

    if not TestObj.tend():
        sys.exit(FAIL)
    sys.exit(PASS)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_replicate_success():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    rep_name = read_var("LSM_VOL_REP_NAME")
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        # TODO: Replicate to different pool
        rep_types = ["CLONE", "COPY", "MIRROR_ASYNC", "MIRROR_SYNC"]
        if 'ontap' in config['protocol']:
            rep_types = ["CLONE"]  # only clone is supported on ontap yet


        for rep_type in rep_types:
            ret, data = atomic_run("Replicating volume %s to %s with type %s with protocol %s." %
                                   (vol_id, rep_name, rep_type, config['protocol']),
                                   command=lsm.volume_replicate,
                                   name=rep_name,
                                   vol=vol_id,
                                   pool=pool_id,
                                   rep_type=rep_type,
                                   return_output=True,
                                   errors=errors)
            rep_id = [line.split()[0].strip() for line in data.splitlines() if rep_name in line][0]
            atomic_run("Removing volume %s with protocol %s." % (rep_name, config['protocol']),
                       command=lsm.volume_delete,
                       vol=rep_id,
                       force=True,
                       errors=errors)
    return errors


def volume_replicate_fail():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    rep_name = read_var("LSM_VOL_REP_NAME")
    rep_type = "CLONE"

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail replicating volume without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.volume_replicate},
            {'message': "Trying to fail replicating volume without volume with protocol %s" %
                        config['protocol'], 'name': rep_name, 'rep_type': rep_type, 'command': lsm.volume_replicate},
            {'message': "Trying to fail replicating volume without name with protocol %s" %
                        config['protocol'], 'vol': vol_id, 'rep_type': rep_type, 'command': lsm.volume_replicate},
            {'message': "Trying to fail replicating volume without rep-type with protocol %s" %
                        config['protocol'], 'vol': vol_id, 'name': rep_name, 'command': lsm.volume_replicate},
            {'message': "Trying to fail replicating volume with wrong volume with protocol %s" %
                        config['protocol'], 'name': rep_name, 'rep_type': rep_type, 'vol': "WRONG",
             'command': lsm.volume_replicate},
            {'message': "Trying to fail replicating volume with wrong type with protocol %s" %
                        config['protocol'], 'name': rep_name, 'rep_type': "WRONG", 'vol': vol_id,
             'command': lsm.volume_replicate},
            {'message': "Trying to fail replicating volume with wrong pool with protocol %s" %
                        config['protocol'], 'name': rep_name, 'rep_type': rep_type, 'vol': vol_id, 'pool': "WRONG",
             'command': lsm.volume_replicate},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_replicate_name_conflict():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    rep_name = read_var("LSM_VOL_NAME")
    pool_id = read_var("LSM_POOL_ID_VOLUME")
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        for rep_type in ["CLONE", "COPY", "MIRROR_ASYNC", "MIRROR_SYNC"]:
            atomic_run("Replicating volume %s to %s with type %s to hit NAME_CONFLICT with protocol %s." %
                       (vol_id, rep_name, rep_type, config['protocol']),
                       command=lsm.volume_replicate,
                       name=rep_name,
                       vol=vol_id,
                       pool=pool_id,
                       rep_type=rep_type,
                       return_output=True,
                       expected_ret=4,
                       errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_replicate_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_replicate_fail()
        errs += volume_replicate_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, clean_var
from os import environ


def remove_fs():
    print("INFO: Removing AG.")
    errors = []

    var_name = "LSM_" + environ["fmf_ag_id"]
    ag_id = read_var(var_name)

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    atomic_run("Removing AG '%s'." % ag_id,
               command=lsm.access_group_delete,
               ag=ag_id,
               force=True,
               errors=errors)

    atomic_run("Removing var %s" % var_name,
               command=clean_var,
               var=var_name,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = remove_fs()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.persistent_vars import write_var
from stqe.host.atomic_run import parse_ret, atomic_run
from libsan.host.mp import mp_stop_service, flush_all, mp_service_name
from libsan.host.linux import is_installed, is_service_running, service_stop


def stop_multipath():
    errors = []
    mp_socket_process = "multipathd.socket"

    if not is_installed('device-mapper-multipath'):
        print("WARN: Skipping setup! Package device-mapper-multipath is not installed!")
        return errors

    if not is_service_running(mp_socket_process) and not is_service_running(mp_service_name()):
        print("WARN: Skipping setup! Services multipathd.service and multipathd.socket are dead!")
        atomic_run("Writing var START_MULTIPATH",
                   command=write_var,
                   var={"START_MULTIPATH": 1},
                   errors=errors)

        return errors

    atomic_run("Stopping service multipathd",
               command=mp_stop_service,
               errors=errors
               )

    atomic_run("Flushing all",
               command=flush_all,
               errors=errors
               )

    if is_service_running(mp_service_name()):
        msg = "FAIL: Service %s is running!" % mp_service_name()
        print(msg)
        errors.append(msg)
        return errors

    if is_service_running(mp_socket_process):
        atomic_run("Stopping service %s" % mp_socket_process,
                   service_name=mp_socket_process,
                   command=service_stop,
                   errors=errors
                   )

        if is_service_running(mp_socket_process):
            msg = "FAIL: Service %s is running!" % mp_socket_process
            print(msg)
            errors.append(msg)
            return errors

    return errors


if __name__ == "__main__":
    errs = stop_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, read_env


def unbind_pool():
    errors = []
    binding_method = None
    stratis = Stratis()

    pool_name = read_var(read_env("fmf_pool_name"))
    try:
        binding_method = read_env("fmf_binding_method")
    except KeyError:
        pass

    atomic_run("Creating stratis pool %s." % pool_name,
               command=stratis.pool_unbind,
               pool_name=pool_name,
               binding_method=binding_method,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = unbind_pool()
    exit(parse_ret(errs))



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

#
# Bugs related:
#   BZ1468727 - System crash on rport removal when rport's scsi_target structure is only partialy initialized
#

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.scsi as scsi
import libsan.host.mp as mp
import libsan.host.linux as linux
import libsan.host.fc as fc
import libsan.host.fcoe as fcoe
import libsan.sanmgmt as sanmgmt
import stqe.host.nfs_lock as nfs_lock
import stqe.host.tc
import re
import sys
import os
import subprocess
import argparse
import traceback

TestObj = None

LOCK_TYPE = "shared"

clean_up_info = {}
lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = "date \"+%Y-%m-%d %H:%M:%S\""
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")

    print("[%s] %s" % (stdout, string))

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """
    """
    global clean_up_info

    if not clean_up_info:
        return True

    if "rescan_pid" in clean_up_info:
        linux.kill_pid(clean_up_info["rescan_pid"])

    # TODO: Bring link back up
    if ("obj_sanmgmt" in clean_up_info and
            "h_wwpn" in clean_up_info):
        clean_up_info["obj_sanmgmt"].phy_link_trigger(addr=clean_up_info["test_wwpn"],
                                                      action="UP")
        # give some time for r_port to be reconnected
        linux.sleep(10)

    # Restore r_port setting back to original values
    r_port = None
    if "r_port" in clean_up_info:
        r_port = clean_up_info["r_port"]
    if r_port and "fast_io_fail_tmo" in clean_up_info:
        print("INFO: Restoring initial fast_io_fail_tmo value")
        fc.set_value_rport_parameter(r_port, "fast_io_fail_tmo",
                                     clean_up_info["fast_io_fail_tmo"])
    if r_port and "dev_loss_tmo" in clean_up_info:
        print("INFO: Restoring initial dev_loss_tmo value")
        fc.set_value_rport_parameter(r_port, "dev_loss_tmo",
                                     clean_up_info["dev_loss_tmo"])
    mp.mp_start_service()

    if "lock_obj" in clean_up_info:
        clean_up_info["lock_obj"].release_lock(LOCK_TYPE)
    _print("INFO: Clean up done.")
    clean_up_info = {}
    return True


def _compare_rport_status(r_port, expected_state):
    r_port_state = fc.get_value_rport_parameter(r_port, "port_state")
    if not r_port_state:
        print("FAIL: Could not get r_port %s state" % r_port)
        return False
    if r_port_state != expected_state:
        print("FAIL: expected r_port to be with state %s, but it is %s" %
              (expected_state, r_port_state))
        return False
    return True


def dev_loss_tmo_test(mpath_name):
    #    global clean_up_info, TestObj

    print("INFO: Starting test run for sys/oscillate_port dev_loss_tmo = %s" % dev_loss_tmo)
    if "obj_sanmgmt" not in clean_up_info:
        print("FAIL: Could not get obj_sanmgmt")
        return False

    obj_sanmgmt = clean_up_info["obj_sanmgmt"]

    # Force some IO to make sure there is an active path
    run("dd if=/dev/zero of=/dev/mapper/%s count=512" % mpath_name)

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    mp.multipath_show(mpath_name)
    active_disks = mp.mpath_get_active_disk(mpath_name)
    if not active_disks:
        print("FAIL: Could not find the active path from %s" % mpath_name)
        return False
    # We will test only one of active disks, most common mpath config just have 1 active disk each time anyway
    active_disk = active_disks[0]

    # Stop only after getting the active disk
    print("INFO: Stoping multipathd as it can also control dev_loss_tmo")
    if not mp.mp_stop_service():
        print("FAIL: Could not stop multipath service")
        return False

    clean_up_info["active_disk"] = active_disk

    required_mp_info = ["wwid", "disk"]
    for req in required_mp_info:
        if req not in mp_info:
            print("FAIL: mpath info does not provide %s that is required" % req)
            return False
    wwid = mp_info["wwid"]
    if active_disk not in mp_info["disk"].keys():
        print("FAIL: Could not find %s on mpath %s" % (active_disk, mpath_name))
        return False
    disk_info = mp_info["disk"][active_disk]
    # print "DEBUG: Disk info"
    # print disk_info

    required_disk_info = ["h_wwpn", "t_wwpn"]
    for req in required_disk_info:
        if req not in disk_info.keys():
            print("FAIL: disk does not provide %s that is required" % req)
            return False
    h_wwpn = disk_info["h_wwpn"]
    t_wwpn = disk_info["t_wwpn"]
    host_id = disk_info["host_id"]
    r_port = fc.rport_of_h_wwpn_t_wwpn(h_wwpn=h_wwpn, t_wwpn=t_wwpn)
    if not r_port:
        print("FAIL: Could not find r_port_id")
        return False

    print("INFO:  Choose SCSI disk from mpath: %s" % mpath_name)
    print("\t\tHost ID    : %s" % host_id)
    print("\t\tHost WWPN  : %s" % h_wwpn)
    print("\t\tTarget WWPN: %s" % t_wwpn)
    print("\t\tWWID       : %s" % wwid)
    print("\t\tSCSI disk  : %s" % active_disk)
    print("\t\tFC rport   : %s" % r_port)

    if port_type == "fc_host":
        test_wwpn = h_wwpn
    else:
        test_wwpn = t_wwpn

    if phy_switch:
        if not obj_sanmgmt.get_physw_self(test_wwpn):
            TestObj.tskip("WWPN '%s' is not connected to phy switch" % test_wwpn)
            _clean_up()
            TestObj.tend()
            sys.exit(2)
    else:
        if not obj_sanmgmt.get_sw_self(test_wwpn):
            TestObj.tskip("WWPN '%s' is not connected to SAN switch" % test_wwpn)
            _clean_up()
            TestObj.tend()
            sys.exit(2)

    # Save wwpn to bring it up on clean up
    clean_up_info["test_wwpn"] = test_wwpn

    clean_up_info["r_port"] = r_port

    init_dev_loss_tmo = fc.get_value_rport_parameter(r_port, "dev_loss_tmo")
    if not init_dev_loss_tmo:
        print("FAIL: Could not get dev_loss_tmo for %s" % r_port)
        return False
    init_fast_io_fail_tmo = fc.get_value_rport_parameter(r_port, "fast_io_fail_tmo")
    if not init_fast_io_fail_tmo:
        print("FAIL: Could not get fast_io_fail_tmo for %s" % r_port)
        return False
    clean_up_info["dev_loss_tmo"] = init_dev_loss_tmo
    clean_up_info["fast_io_fail_tmo"] = init_fast_io_fail_tmo
    print("INFO: Original fast_io_fail_tmo is %s" % init_fast_io_fail_tmo)
    print("INFO: Original dev_loss_tmo is %s" % init_dev_loss_tmo)

    if not fc.set_value_rport_parameter(r_port, "fast_io_fail_tmo", "off"):
        print("FAIL: Could not set fast_io_fail_tmo of %s to %s" % (r_port, "off"))
        return False
    print("INFO: Active SCSI disk %s(%s) fast_io_fail_tmo is set to 'off'" %
          (active_disk, r_port))

    if not fc.set_value_rport_parameter(r_port, "dev_loss_tmo", dev_loss_tmo):
        print("FAIL: Could not set dev_loss_tmo of %s to %s" % (r_port, dev_loss_tmo))
        return False
    print("INFO: Active SCSI disk %s(%s) dev_loss_tmo is set to '%s'" %
          (active_disk, r_port, dev_loss_tmo))

    if not _compare_rport_status(r_port, "Online"):
        print("FAIL: r_port %s is already down" % r_port)
        return False

    # Force scsi rescan on background
    # Delete the disks from the server, to force proper rescan of target
    pid = os.fork()
    if pid == 0:
        # I'm child
        while True:
            scsi.rescan_host(host_id, verbose=False)
            targets = fc.get_fc_host_rport_targets(host_id, r_port)
            if not targets:
                continue
            for target in targets:
                devs = fc.get_fc_host_rport_target_devices(host_id, r_port, target)
                if not devs:
                    continue
                for dev in devs:
                    disk = scsi.get_scsi_disk_name(dev)
                    if not disk:
                        continue
                    scsi.delete_disk(disk)
                    # _print("INFO: deleted disk %s" % disk)

    else:
        print("INFO: rescanning host%s on background with PID %s" % (host_id, pid))
        # Bring host port down on switch
        if phy_switch:
            if not obj_sanmgmt.phy_link_oscillate(test_wwpn, min_uptime=min_link_up_tmo,
                                                  max_uptime=max_link_up_tmo,
                                                  min_downtime=min_link_down_tmo,
                                                  max_downtime=max_link_down_tmo,
                                                  count=rounds):
                print("FAIL: To oscillate phy switch port")
                linux.kill_pid(pid)
                return False
        else:
            if not obj_sanmgmt.link_oscillate(test_wwpn, min_uptime=min_link_up_tmo,
                                              max_uptime=max_link_up_tmo,
                                              min_downtime=min_link_down_tmo,
                                              max_downtime=max_link_down_tmo,
                                              count=rounds):
                print("FAIL: To oscillate SAN switch port")
                linux.kill_pid(pid)
                return False
        linux.kill_pid(pid)

    # No need to restore port on clean_up
    del clean_up_info["test_wwpn"]

    current_dev_loss_tmo = fc.get_value_rport_parameter(r_port, "dev_loss_tmo")
    if int(current_dev_loss_tmo) != dev_loss_tmo:
        _print("FAIL: dev_loss_tmo is %s, but expected to be: %s" % (current_dev_loss_tmo, dev_loss_tmo))
        return False

    # Wait port to be restored
    current_port_staus = "Not Present"
    max_wait = 5
    while max_wait > 0:
        print("INFO: Waiting rport came online, timeout: %d" % max_wait)
        linux.sleep(1)
        linux.wait_udev()
        max_wait -= 1
        r_port = fc.rport_of_h_wwpn_t_wwpn(h_wwpn=h_wwpn, t_wwpn=t_wwpn)
        if not r_port:
            # port still not up
            continue
        if not _compare_rport_status(r_port, "Online"):
            # port still not up
            continue
        current_port_staus = "Online"
        break

    if max_wait == 0 and current_port_staus != "Online":
        print("FAIL: max_link_tmo expired and r_port was not restored")
        return False
    if current_port_staus != "Online":
        # Should not be possible to enter here
        print("FAIL: Did not reach time out, but port still not Online")
        return False

    return True


def execute_test(mpath_name):
    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        TestObj.tfail("Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        clean_up_info["lock_obj"] = lock_obj
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            return False
        _print("INFO: Good, got NFS lock. Start testing...")

    if not dev_loss_tmo_test(mpath_name):
        TestObj.tfail("oscillate_port test on %s" % mpath_name)
        return False
    _print("PASS: oscillate_port test")
    return True


def main():
    """
    """
    global TestObj, LOCK_TYPE
    global min_link_up_tmo, max_link_up_tmo, min_link_down_tmo, max_link_down_tmo
    global rounds, dev_loss_tmo, port_type, phy_switch
    PASS = 0
    FAIL = 1
    SKIP = 2

    parser = argparse.ArgumentParser(
        description='Test remote port removal when dev_loss happens')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to use", metavar="mpath")
    parser.add_argument("--min-link-up-tmo", required=False, dest="min_link_up_tmo", default=0,
                        help="Minimum amount of time to wait with link up", type=float, metavar="number")
    parser.add_argument("--max-link-up-tmo", required=False, dest="max_link_up_tmo", default=1,
                        help="Maximum amount of time to wait with link up", type=float, metavar="number")
    parser.add_argument("--min-link-down-tmo", required=False, dest="min_link_down_tmo", default=1,
                        help="Minimum amount of time to wait with link up", type=float, metavar="number")
    parser.add_argument("--max-link-down-tmo", required=False, dest="max_link_down_tmo", default=2,
                        help="Maximum amount of time to wait with link up", type=float, metavar="number")
    parser.add_argument("--dev-loss-tmo", required=False, dest="dev_loss_tmo", default=1,
                        help="Device loss timeout", type=int, metavar="number")
    parser.add_argument("--rounds", required=False, dest="rounds", default=500,
                        help="How many times the link should oscillate", type=int, metavar="number")
    parser.add_argument("--port-type", required=True, dest="port_type", default=None,
                        help="Port type", choices=["fc_host", "fc_target"])
    parser.add_argument("--phy-switch", required=False, dest="phy_switch", action="store_true",
                        help="Uses phy switch instead of SAN switch", default=False)

    args = parser.parse_args()

    TestObj = stqe.host.tc.TestClass()

    os_version = linux.dist_ver()
    os_version_minor = linux.dist_ver_minor()

    #Skip running oscialte_port.py on RHEL-7 and RHEL-8 until BZ1527648 is resolved
    if linux.dist_name() == "RHEL":
        #if ((os_version == 8 and os_version_minor < 1) or
                #(os_version == 7 and os_version_minor < 8) or
                #(os_version == 6 and os_version_minor < 11)):
        if ((os_version == 8) or
                (os_version == 7) or
                (os_version == 6)):
            TestObj.tskip("oscillate_port test do not run on %s.%s" % (os_version, os_version_minor))
            TestObj.tend()
            sys.exit(SKIP)

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    min_link_up_tmo = args.min_link_up_tmo
    max_link_up_tmo = args.max_link_up_tmo
    min_link_down_tmo = args.min_link_down_tmo
    max_link_down_tmo = args.max_link_down_tmo
    phy_switch = args.phy_switch
    rounds = args.rounds
    dev_loss_tmo = args.dev_loss_tmo
    port_type = args.port_type
    if port_type == "fc_target":
        LOCK_TYPE = "exclusive"

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        # If no mpath is given, make sure soft fcoe is configured
        # Because test will try to find mpath devices
        fcoe.setup_soft_fcoe()

        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    for mpath_name in mpath_name_list:
        try:
            execute_test(mpath_name)
        except Exception as e:
            print(e)
            TestObj.tfail("Could not execute test on %s" % mpath_name)
            traceback.print_exc()
        _clean_up()

    if not TestObj.tend():
        sys.exit(FAIL)
    sys.exit(PASS)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_export_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    export_path = read_var("LSM_FS_EXPORT_PATH")
    rw_host = read_var("LSM_RW_HOST")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Exporting FS '%s'" % fs_id,
                               command=lsm.fs_export,
                               export_path=export_path,
                               rw_host=rw_host,
                               fs=fs_id,
                               return_output=True,
                               errors=errors)
        export_id = [line.split()[0].strip() for line in data.splitlines() if export_path in line][0]
        atomic_run("Removing export %s with protocol %s." % (export_id, config['protocol']),
                   command=lsm.fs_unexport,
                   export=export_id,
                   force=True,
                   errors=errors)

        # FIXME: Add more paramethers

    return errors


def fs_export_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail exporting FS without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.fs_export},
            {'message': "Trying to fail exporting FS with wrong 'fs' 'wrong' with protocol %s" % config['protocol'],
             'command': lsm.fs_export, 'fs': "wrong"},
            {'message': "Trying to fail exporting FS with 'fs' name instead of ID with protocol %s" % config[
                'protocol'], 'command': lsm.fs_export, 'fs': fs_name},
        ]

        for argument in arguments:
            if argument['command'] == lsm.fs_export:
                argument['expected_ret'] = 2
            atomic_run(errors=errors,
                       return_output=True,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_export_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_export_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


# fs_delete_success is part of fs_create_success() in this same directory

def fs_delete_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail deleting FS without name with protocol %s" % config['protocol'],
             'command': lsm.fs_delete},
            {'message': "Trying to fail deleting FS with WRONG name with protocol %s" % config['protocol'],
             'command': lsm.fs_delete, 'fs': "WRONG"},
            {'message': "Trying to fail deleting FS with name instead of ID name with protocol %s" % config['protocol'],
             'command': lsm.fs_delete, 'fs': fs_name},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 2:
        errs = fs_delete_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI
from libsan.host.cmdline import run
from hashlib import sha1
from json import load


def saveconfig_cleanup():
    errors = []

    atomic_run("Removing backups from /etc/target/backup",
               cmd="rm -f /etc/target/backup/*",
               command=run,
               errors=errors
               )

    atomic_run("Removing saveconfig.json",
               cmd="rm -f /etc/target/saveconfig.json",
               command=run,
               errors=errors
               )

    return errors


def compare_backup():
    errors = []
    target = TargetCLI()

    for i in range(2):
        atomic_run("Saving configuration...",
                   command=target.saveconfig,
                   errors=errors
                   )

    with open("/etc/target/saveconfig.json", "r") as last_save:
        saveconfig = load(last_save)
        hash_saveconfig = sha1(repr(saveconfig).encode('utf-8')).hexdigest()

    ret, data = atomic_run("Listing /etc/target/backup/ directory",
                           return_output=True,
                           cmd="ls -lt /etc/target/backup/",
                           command=run,
                           errors=errors
                           )

    lines = data.split()
    backup_file = "Does_not_exist"
    for line in lines:
        if "saveconfig" in line:
            backup_file = line
            break
    with open("/etc/target/backup/" + backup_file, "r") as backup:
        backup_config = load(backup)
        hash_backup = sha1(repr(backup_config).encode('utf-8')).hexdigest()

    if not hash_saveconfig == hash_backup:
        print("FAIL: Backup file is different than last saved configuration")
        errors.append("FAIL: Backup file is different than last saved configuration")

    return errors


if __name__ == "__main__":
    errs = compare_backup()
    errs += saveconfig_cleanup()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def cleanup_crypt():
    disk = read_var("CRYPTED_DISK")
    print("INFO: Cleaning up dm-crypt from device %s." % disk)
    errors = []

    crypted_device = read_var("VDO_DEVICE")

    atomic_run(message="Closing LUKS device %s." % crypted_device,
               command=run,
               cmd="cryptsetup luksClose %s" % crypted_device,
               errors=errors)

    atomic_run(message="Removing LUKS signature from disk %s." % disk,
               command=run,
               cmd="wipefs --all %s" % disk,
               errors=errors)

    atomic_run("Cleaning var CRYPTED_DISK",
               command=clean_var,
               var="CRYPTED_DISK",
               errors=errors)

    atomic_run("Writing var VDO_DEVICE",
               command=write_var,
               var={'VDO_DEVICE': "%s" % disk},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_crypt()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lio import TargetCLI
from stqe.host.persistent_vars import read_var
from stqe.host.atomic_run import atomic_run, parse_ret


def delete_iscsi():
    errors = []
    target = TargetCLI(path="/iscsi")
    target_iqn = read_var("TARGET_IQN")

    atomic_run("Deleting iscsi target: %s" % target_iqn,
               wwn=target_iqn,
               command=target.delete,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = delete_iscsi()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ


def local_disk_list_success():
    errors = []
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run("Using LSM to list local disks with protocol %s" % config['protocol'],
                   command=lsm.local_disk_list,
                   errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = local_disk_list_success()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, read_env, write_var
import time


def create_cache_disk():
    errors = []
    stratis = Stratis()
    pool_name = read_env('fmf_pool_name')

    id = ""
    try:
        id = "_" + str(read_env("fmf_id"))
        pool_name = pool_name + id
    except KeyError:
        pass

    previous = ""
    try:
        previous = int(id[1:]) - 1
        if previous == 1:
            previous = ""
        else:
            previous = "_" + str(previous)
    except ValueError:
        pass
    print("previous is %s, id is %s," % (previous, id))
    blockdevs = read_var("STRATIS_FREE")

    if "init_cache" in read_env("fmf_name"):
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        if blockdevs:
            cache_disk = blockdevs.pop()
            atomic_run("Writing var STRATIS_FREE",
                       command=write_var,
                       var={'STRATIS_FREE': blockdevs},
                       errors=errors)

    time.sleep(2)
    atomic_run(message="Triggering udev",
               command=run,
               cmd="udevadm trigger; udevadm settle",
               errors=errors)

    atomic_run("statis pool init cache %s." % pool_name,
               command=stratis.pool_init_cache,
               pool_name=pool_name,
               blockdevs=cache_disk,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_cache_disk()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.scsi import get_free_disks
from libsan.host.mp import mpath_name_of_wwid, remove_mpath, is_multipathd_running
from stqe.host.persistent_vars import write_var, read_var, read_env
from libsan.host.linux import is_service_running
from libsan.host.lio import TargetCLI


def setup_local_disks(number_of_disks):
    msg = "INFO: Getting local disks."
    if number_of_disks:
        msg += " Trying to get %s disks" % number_of_disks
    print(msg)
    errors = []

    disks = atomic_run(message="Getting free disks",
                       command=get_free_disks,
                       errors=errors)
    # multiple_device test needs at least 4 disks
    if disks is None:
        disks = {}

    if len(disks.keys()) < number_of_disks:
        msg = f"INFO: Found only {len(disks.keys())} disks, need {number_of_disks} disks. " \
              f"Creating "
        print(msg)
        wwn = "naa.50014054c1441891"
        stratis_device_size = "10G"
        name = "stratis-lun-"

        try:
            stratis_device_size = read_env("fmf_loopback_size")
        except KeyError:
            pass

        for file in range(number_of_disks - len(disks.keys())):
            target = TargetCLI(path="/backstores/fileio")
            atomic_run("Creating fileio backstore",
                       command=target.create,
                       name=f"{name}{file}",
                       file_or_dev=f"/home/{name}{file}.img",
                       size=stratis_device_size,
                       errors=errors,
                       )

        target.path = "/loopback"
        atomic_run("Creating loopback",
                   wwn=wwn,
                   command=target.create,
                   errors=errors
                   )

        target.path = "/loopback/" + wwn + "/" + "luns"

        for lun in range(number_of_disks - len(disks.keys())):
            atomic_run("Creating lun using fileio backstore",
                       command=target.create,
                       storage_object=f"/backstores/fileio/{name}{lun}",
                       errors=errors,
                       )

        atomic_run("Writing var STRATIS_LOOPBACK_WWN",
                   command=write_var,
                   var={'STRATIS_LOOPBACK_WWN': wwn},
                   errors=errors)

        atomic_run("Writing var STRATIS_LUN_NAME",
                   command=write_var,
                   var={'STRATIS_LUN_NAME': name},
                   errors=errors)

        atomic_run("Writing var STRATIS_NUMBER_OF_LUNS",
                   command=write_var,
                   var={'STRATIS_NUMBER_OF_LUNS': number_of_disks - len(disks.keys())},
                   errors=errors)

        if is_multipathd_running():
            lio_disks = atomic_run(message="Getting free disks",
                                   exclude_mpath_device=False,
                                   command=get_free_disks,
                                   filter={"vendor": "LIO-ORG"},
                                   errors=errors)

            # Try to remove mpath from lio disks
            for disk in lio_disks.keys():
                mpath_name = mpath_name_of_wwid(lio_disks[disk]["wwid"])
                if mpath_name:
                    print(f"INFO: Trying to remove mpath {mpath_name}")
                    remove_mpath(mpath_name)

        disks = atomic_run(message="Getting free disks",
                           command=get_free_disks,
                           errors=errors)
    else:
        atomic_run("Writing var STRATIS_NUMBER_OF_LUNS",
                   command=write_var,
                   var={'STRATIS_NUMBER_OF_LUNS': 0},
                   errors=errors)

    disks = disks.keys()
    disk_paths = ["/dev/" + j for j in disks]
    blockdevs = read_var("STRATIS_DEVICE")

    if blockdevs:
        # backup the previous devices
        atomic_run("Writing var STRATIS_DEVICE_BACKUP",
                   command=write_var,
                   var={'STRATIS_DEVICE_BACKUP': " ".join(blockdevs)},
                   errors=errors)
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        disk_paths += [x for x in blockdevs if x not in blockdevs]

    print("Using these blockdevs: %s" % " ".join(disk_paths))
    for disk in disk_paths:
        atomic_run("Zeroing superblock of disk %s." % disk,
                   command=run,
                   cmd="dd if=/dev/zero of=%s bs=1M count=10" % disk,
                   errors=errors)
        if is_service_running("multipathd"):
            atomic_run("remove multipath superblock of disk %s." % disk,
                       command=run,
                       cmd="multipath -W %s" % disk,
                       errors=errors)

    atomic_run("Writing var STRATIS_AVAILABLE_DEVICES",
               command=write_var,
               var={'STRATIS_AVAILABLE_DEVICES': disk_paths},
               errors=errors)

    atomic_run("Writing var STRATIS_DEVICE",
               command=write_var,
               var={'STRATIS_DEVICE': disk_paths},
               errors=errors)

    atomic_run(message="Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    try:
        number_of_disks = read_env("fmf_number_of_disks")
    except KeyError:
        number_of_disks = None
    errs = setup_local_disks(number_of_disks)
    exit(parse_ret(errs))



#!/usr/bin/python

from __future__ import absolute_import, division, print_function, unicode_literals
import stqe.host.tc
from libsan.host.cmdline import run
from libsan.host.linux import install_package
import sys
import re

TC = None


def start_test():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    print ("INFO: Making sure tools provided by device_mapper_persistent_data "
           "are not linked to /usr")

    # Paths where we should have no libraries linked from
    lib_paths = ["/usr/"]

    package = "device-mapper-persistent-data"
    install_package(package)
    # Get all tools that we need to check
    ret, output = run("rpm -ql %s | grep \"sbin/\"" % package,
                      return_output=True,
                      verbose=False)
    tools = output.split("\n")

    for tool in tools:
        error = 0
        for lib_path in lib_paths:
            print("INFO: Checking if %s linked to libraries at %s" % (tool, lib_path))
            ret, linked_lib = run("ldd %s" % tool, return_output=True)
            if ret != 0:
                TC.tfail("Could not list dynamically libraries for %s" % (tool))
                error += 1
            else:
                # The command executed sucessfuly
                # check if any library linked is from lib_path
                links = linked_lib.split("\n")
                for link in links:
                    if re.match(".*%s.*" % lib_path, link):
                        TC.tfail("%s is linked to %s" % (tool, link))
                        error += 1

        if error == 0:
            TC.tpass("%s is not linked to /usr/" % tool)

    return True


def main():
    global TC

    start_test()

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from time import sleep
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def cleanup_free_disks():
    print("INFO: Cleaning up free disks to previous state.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    stratis_number_of_luns = read_var("STRATIS_NUMBER_OF_LUNS")
    if stratis_number_of_luns != 0:
        path = "/home/"
        wwn = read_var("STRATIS_LOOPBACK_WWN")
        stratis_fileio_name = read_var("STRATIS_LUN_NAME")

        atomic_run("Deleting loopback",
                   cmd=f"targetcli /loopback delete {wwn}",
                   command=run,
                   errors=errors
                   )

        for lun in range(stratis_number_of_luns):
            atomic_run(f"Deleting backstore {stratis_fileio_name}{lun}",
                       cmd=f"targetcli /backstores/fileio delete {stratis_fileio_name}{lun}",
                       command=run,
                       errors=errors
                       )

            atomic_run(f"Removing file from {path}",
                       cmd=f"rm -f {path}{stratis_fileio_name}{lun}.img",
                       command=run,
                       errors=errors
                       )

        atomic_run("Cleaning var STRATIS_LOOPBACK_WWN",
                   command=clean_var,
                   var="STRATIS_LOOPBACK_WWN",
                   errors=errors)

        atomic_run("Cleaning var STRATIS_LUN_NAME",
                   command=clean_var,
                   var="STRATIS_LUN_NAME",
                   errors=errors)

    atomic_run("Cleaning var STRATIS_DEVICE",
               command=clean_var,
               var="STRATIS_DEVICE",
               errors=errors)

    atomic_run("Cleaning var STRATIS_AVAILABLE_DEVICES",
               command=clean_var,
               var="STRATIS_AVAILABLE_DEVICES",
               errors=errors)

    backup = read_var("STRATIS_DEVICE_BACKUP")
    if backup:
        atomic_run("Cleaning var STRATIS_DEVICE_BACKUP",
                   command=clean_var,
                   var="STRATIS_DEVICE_BACKUP",
                   errors=errors)

        atomic_run("Writing var STRATIS_DEVICE",
                   command=write_var,
                   var={'STRATIS_DEVICE': backup},
                   errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_free_disks()
    exit(parse_ret(errs))
